@@ -1,10 +1,11 @@
 package network
 
 import (
-	"github.com/containers/podman/v3/cmd/podman/common"
-	"github.com/containers/podman/v3/cmd/podman/inspect"
-	"github.com/containers/podman/v3/cmd/podman/registry"
-	"github.com/containers/podman/v3/pkg/domain/entities"
+	"github.com/containers/common/libnetwork/types"
+	"github.com/containers/podman/v4/cmd/podman/common"
+	"github.com/containers/podman/v4/cmd/podman/inspect"
+	"github.com/containers/podman/v4/cmd/podman/registry"
+	"github.com/containers/podman/v4/pkg/domain/entities"
 	"github.com/spf13/cobra"
 )
 
@@ -12,8 +13,8 @@ var (
 	networkinspectDescription = `Inspect network`
 	networkinspectCommand     = &cobra.Command{
 		Use:               "inspect [options] NETWORK [NETWORK...]",
-		Short:             "Displays the raw CNI network configuration for one or more networks.",
-		Long:              networkinspectDescription,
+		Long:              "Displays the network configuration for one or more networks.",
+		Short:             networkinspectDescription,
 		RunE:              networkInspect,
 		Example:           `podman network inspect podman`,
 		Args:              cobra.MinimumNArgs(1),
@@ -32,10 +33,10 @@ func init() {
 
 	formatFlagName := "format"
 	flags.StringVarP(&inspectOpts.Format, formatFlagName, "f", "", "Pretty-print network to JSON or using a Go template")
-	_ = networkinspectCommand.RegisterFlagCompletionFunc(formatFlagName, common.AutocompleteFormat(nil))
+	_ = networkinspectCommand.RegisterFlagCompletionFunc(formatFlagName, common.AutocompleteFormat(&types.Network{}))
 }
 
 func networkInspect(_ *cobra.Command, args []string) error {
-	inspectOpts.Type = inspect.NetworkType
+	inspectOpts.Type = common.NetworkType
 	return inspect.Inspect(args, *inspectOpts)
 }
