@@ -2,14 +2,12 @@ package libpod
 
 import (
 	"bufio"
-	"bytes"
 	"context"
+	"errors"
 	"fmt"
+	"math/rand"
 	"os"
-	"os/exec"
 	"path/filepath"
-	"regexp"
-	"strconv"
 	"strings"
 	"sync"
 	"syscall"
@@ -19,42 +17,30 @@ import (
 
 	"github.com/containers/buildah/pkg/parse"
 	"github.com/containers/common/libimage"
+	"github.com/containers/common/libnetwork/network"
+	nettypes "github.com/containers/common/libnetwork/types"
+	"github.com/containers/common/pkg/cgroups"
 	"github.com/containers/common/pkg/config"
-	"github.com/containers/common/pkg/defaultnet"
 	"github.com/containers/common/pkg/secrets"
 	"github.com/containers/image/v5/pkg/sysregistriesv2"
 	is "github.com/containers/image/v5/storage"
 	"github.com/containers/image/v5/types"
-	"github.com/containers/podman/v3/libpod/define"
-	"github.com/containers/podman/v3/libpod/events"
-	"github.com/containers/podman/v3/libpod/lock"
-	"github.com/containers/podman/v3/libpod/plugin"
-	"github.com/containers/podman/v3/libpod/shutdown"
-	"github.com/containers/podman/v3/pkg/cgroups"
-	"github.com/containers/podman/v3/pkg/rootless"
-	"github.com/containers/podman/v3/pkg/systemd"
-	"github.com/containers/podman/v3/pkg/util"
-	"github.com/containers/podman/v3/utils"
+	"github.com/containers/podman/v4/libpod/define"
+	"github.com/containers/podman/v4/libpod/events"
+	"github.com/containers/podman/v4/libpod/lock"
+	"github.com/containers/podman/v4/libpod/plugin"
+	"github.com/containers/podman/v4/libpod/shutdown"
+	"github.com/containers/podman/v4/pkg/rootless"
+	"github.com/containers/podman/v4/pkg/systemd"
+	"github.com/containers/podman/v4/pkg/util"
+	"github.com/containers/podman/v4/utils"
 	"github.com/containers/storage"
 	"github.com/containers/storage/pkg/unshare"
-	"github.com/cri-o/ocicni/pkg/ocicni"
 	"github.com/docker/docker/pkg/namesgenerator"
 	spec "github.com/opencontainers/runtime-spec/specs-go"
-	"github.com/pkg/errors"
 	"github.com/sirupsen/logrus"
 )
 
-const (
-	// conmonMinMajorVersion is the major version required for conmon.
-	conmonMinMajorVersion = 2
-
-	// conmonMinMinorVersion is the minor version required for conmon.
-	conmonMinMinorVersion = 0
-
-	// conmonMinPatchVersion is the sub-minor version required for conmon.
-	conmonMinPatchVersion = 24
-)
-
 // A RuntimeOption is a functional option which alters the Runtime created by
 // NewRuntime
 type RuntimeOption func(*Runtime) error
@@ -81,12 +67,25 @@ type Runtime struct {
 	defaultOCIRuntime      OCIRuntime
 	ociRuntimes            map[string]OCIRuntime
 	runtimeFlags           []string
-	netPlugin              ocicni.CNIPlugin
+	network                nettypes.ContainerNetwork
 	conmonPath             string
 	libimageRuntime        *libimage.Runtime
 	libimageEventsShutdown chan bool
 	lockManager            lock.Manager
 
+	// Worker
+	workerChannel chan func()
+	workerGroup   sync.WaitGroup
+
+	// syslog describes whenever logrus should log to the syslog as well.
+	// Note that the syslog hook will be enabled early in cmd/podman/syslog_linux.go
+	// This bool is just needed so that we can set it for netavark interface.
+	syslog bool
+
+	// doReset indicates that the runtime should perform a system reset.
+	// All Podman files will be removed.
+	doReset bool
+
 	// doRenumber indicates that the runtime should perform a lock renumber
 	// during initialization.
 	// Once the runtime has been initialized and returned, this variable is
@@ -104,7 +103,6 @@ type Runtime struct {
 	// and remains true until the runtime is shut down (rendering its
 	// storage unusable). When valid is false, the runtime cannot be used.
 	valid bool
-	lock  sync.RWMutex
 
 	// mechanism to read and write even logs
 	eventer events.Eventer
@@ -115,6 +113,13 @@ type Runtime struct {
 	secretsManager *secrets.SecretsManager
 }
 
+func init() {
+	// generateName calls namesgenerator.GetRandomName which the
+	// global RNG from math/rand. Seed it here to make sure we
+	// don't get the same name every time.
+	rand.Seed(time.Now().UnixNano())
+}
+
 // SetXdgDirs ensures the XDG_RUNTIME_DIR env and XDG_CONFIG_HOME variables are set.
 // containers/image uses XDG_RUNTIME_DIR to locate the auth file, XDG_CONFIG_HOME is
 // use for the containers.conf configuration file.
@@ -123,7 +128,7 @@ func SetXdgDirs() error {
 		return nil
 	}
 
-	// Setup XDG_RUNTIME_DIR
+	// Set up XDG_RUNTIME_DIR
 	runtimeDir := os.Getenv("XDG_RUNTIME_DIR")
 
 	if runtimeDir == "" {
@@ -134,7 +139,7 @@ func SetXdgDirs() error {
 		}
 	}
 	if err := os.Setenv("XDG_RUNTIME_DIR", runtimeDir); err != nil {
-		return errors.Wrapf(err, "cannot set XDG_RUNTIME_DIR")
+		return fmt.Errorf("cannot set XDG_RUNTIME_DIR: %w", err)
 	}
 
 	if rootless.IsRootless() && os.Getenv("DBUS_SESSION_BUS_ADDRESS") == "" {
@@ -144,14 +149,14 @@ func SetXdgDirs() error {
 		}
 	}
 
-	// Setup XDG_CONFIG_HOME
+	// Set up XDG_CONFIG_HOME
 	if cfgHomeDir := os.Getenv("XDG_CONFIG_HOME"); cfgHomeDir == "" {
 		cfgHomeDir, err := util.GetRootlessConfigHomeDir()
 		if err != nil {
 			return err
 		}
 		if err := os.Setenv("XDG_CONFIG_HOME", cfgHomeDir); err != nil {
-			return errors.Wrapf(err, "cannot set XDG_CONFIG_HOME")
+			return fmt.Errorf("cannot set XDG_CONFIG_HOME: %w", err)
 		}
 	}
 	return nil
@@ -164,8 +169,7 @@ func NewRuntime(ctx context.Context, options ...RuntimeOption) (*Runtime, error)
 	if err != nil {
 		return nil, err
 	}
-	conf.CheckCgroupsAndAdjustConfig()
-	return newRuntimeFromConfig(ctx, conf, options...)
+	return newRuntimeFromConfig(conf, options...)
 }
 
 // NewRuntimeFromConfig creates a new container runtime using the given
@@ -174,10 +178,10 @@ func NewRuntime(ctx context.Context, options ...RuntimeOption) (*Runtime, error)
 // An error will be returned if the configuration file at the given path does
 // not exist or cannot be loaded
 func NewRuntimeFromConfig(ctx context.Context, userConfig *config.Config, options ...RuntimeOption) (*Runtime, error) {
-	return newRuntimeFromConfig(ctx, userConfig, options...)
+	return newRuntimeFromConfig(userConfig, options...)
 }
 
-func newRuntimeFromConfig(ctx context.Context, conf *config.Config, options ...RuntimeOption) (*Runtime, error) {
+func newRuntimeFromConfig(conf *config.Config, options ...RuntimeOption) (*Runtime, error) {
 	runtime := new(Runtime)
 
 	if conf.Engine.OCIRuntime == "" {
@@ -203,25 +207,36 @@ func newRuntimeFromConfig(ctx context.Context, conf *config.Config, options ...R
 	// Overwrite config with user-given configuration options
 	for _, opt := range options {
 		if err := opt(runtime); err != nil {
-			return nil, errors.Wrapf(err, "error configuring runtime")
+			return nil, fmt.Errorf("configuring runtime: %w", err)
 		}
 	}
 
 	if err := shutdown.Register("libpod", func(sig os.Signal) error {
+		// For `systemctl stop podman.service` support, exit code should be 0
+		if sig == syscall.SIGTERM {
+			os.Exit(0)
+		}
 		os.Exit(1)
 		return nil
-	}); err != nil && errors.Cause(err) != shutdown.ErrHandlerExists {
-		logrus.Errorf("Error registering shutdown handler for libpod: %v", err)
+	}); err != nil && !errors.Is(err, shutdown.ErrHandlerExists) {
+		logrus.Errorf("Registering shutdown handler for libpod: %v", err)
 	}
 
 	if err := shutdown.Start(); err != nil {
-		return nil, errors.Wrapf(err, "error starting shutdown signal handler")
+		return nil, fmt.Errorf("starting shutdown signal handler: %w", err)
 	}
 
-	if err := makeRuntime(ctx, runtime); err != nil {
+	if err := makeRuntime(runtime); err != nil {
 		return nil, err
 	}
 
+	runtime.config.CheckCgroupsAndAdjustConfig()
+
+	// If resetting storage, do *not* return a runtime.
+	if runtime.doReset {
+		return nil, nil
+	}
+
 	return runtime, nil
 }
 
@@ -234,10 +249,10 @@ func getLockManager(runtime *Runtime) (lock.Manager, error) {
 		lockPath := filepath.Join(runtime.config.Engine.TmpDir, "locks")
 		manager, err = lock.OpenFileLockManager(lockPath)
 		if err != nil {
-			if os.IsNotExist(errors.Cause(err)) {
+			if errors.Is(err, os.ErrNotExist) {
 				manager, err = lock.NewFileLockManager(lockPath)
 				if err != nil {
-					return nil, errors.Wrapf(err, "failed to get new file lock manager")
+					return nil, fmt.Errorf("failed to get new file lock manager: %w", err)
 				}
 			} else {
 				return nil, err
@@ -253,19 +268,19 @@ func getLockManager(runtime *Runtime) (lock.Manager, error) {
 		manager, err = lock.OpenSHMLockManager(lockPath, runtime.config.Engine.NumLocks)
 		if err != nil {
 			switch {
-			case os.IsNotExist(errors.Cause(err)):
+			case errors.Is(err, os.ErrNotExist):
 				manager, err = lock.NewSHMLockManager(lockPath, runtime.config.Engine.NumLocks)
 				if err != nil {
-					return nil, errors.Wrapf(err, "failed to get new shm lock manager")
+					return nil, fmt.Errorf("failed to get new shm lock manager: %w", err)
 				}
-			case errors.Cause(err) == syscall.ERANGE && runtime.doRenumber:
+			case errors.Is(err, syscall.ERANGE) && runtime.doRenumber:
 				logrus.Debugf("Number of locks does not match - removing old locks")
 
 				// ERANGE indicates a lock numbering mismatch.
 				// Since we're renumbering, this is not fatal.
 				// Remove the earlier set of locks and recreate.
 				if err := os.Remove(filepath.Join("/dev/shm", lockPath)); err != nil {
-					return nil, errors.Wrapf(err, "error removing libpod locks file %s", lockPath)
+					return nil, fmt.Errorf("removing libpod locks file %s: %w", lockPath, err)
 				}
 
 				manager, err = lock.NewSHMLockManager(lockPath, runtime.config.Engine.NumLocks)
@@ -277,26 +292,33 @@ func getLockManager(runtime *Runtime) (lock.Manager, error) {
 			}
 		}
 	default:
-		return nil, errors.Wrapf(define.ErrInvalidArg, "unknown lock type %s", runtime.config.Engine.LockType)
+		return nil, fmt.Errorf("unknown lock type %s: %w", runtime.config.Engine.LockType, define.ErrInvalidArg)
 	}
 	return manager, nil
 }
 
 // Make a new runtime based on the given configuration
 // Sets up containers/storage, state store, OCI runtime
-func makeRuntime(ctx context.Context, runtime *Runtime) (retErr error) {
+func makeRuntime(runtime *Runtime) (retErr error) {
 	// Find a working conmon binary
-	cPath, err := findConmon(runtime.config.Engine.ConmonPath)
+	cPath, err := runtime.config.FindConmon()
 	if err != nil {
 		return err
 	}
 	runtime.conmonPath = cPath
 
+	if runtime.noStore && runtime.doReset {
+		return fmt.Errorf("cannot perform system reset if runtime is not creating a store: %w", define.ErrInvalidArg)
+	}
+	if runtime.doReset && runtime.doRenumber {
+		return fmt.Errorf("cannot perform system reset while renumbering locks: %w", define.ErrInvalidArg)
+	}
+
 	// Make the static files directory if it does not exist
 	if err := os.MkdirAll(runtime.config.Engine.StaticDir, 0700); err != nil {
 		// The directory is allowed to exist
-		if !os.IsExist(err) {
-			return errors.Wrap(err, "error creating runtime static files directory")
+		if !errors.Is(err, os.ErrExist) {
+			return fmt.Errorf("creating runtime static files directory: %w", err)
 		}
 	}
 
@@ -308,9 +330,9 @@ func makeRuntime(ctx context.Context, runtime *Runtime) (retErr error) {
 	// package.
 	switch runtime.config.Engine.StateType {
 	case config.InMemoryStateStore:
-		return errors.Wrapf(define.ErrInvalidArg, "in-memory state is currently disabled")
+		return fmt.Errorf("in-memory state is currently disabled: %w", define.ErrInvalidArg)
 	case config.SQLiteStateStore:
-		return errors.Wrapf(define.ErrInvalidArg, "SQLite state is currently disabled")
+		return fmt.Errorf("SQLite state is currently disabled: %w", define.ErrInvalidArg)
 	case config.BoltDBStateStore:
 		dbPath := filepath.Join(runtime.config.Engine.StaticDir, "bolt_state.db")
 
@@ -320,13 +342,27 @@ func makeRuntime(ctx context.Context, runtime *Runtime) (retErr error) {
 		}
 		runtime.state = state
 	default:
-		return errors.Wrapf(define.ErrInvalidArg, "unrecognized state type passed (%v)", runtime.config.Engine.StateType)
+		return fmt.Errorf("unrecognized state type passed (%v): %w", runtime.config.Engine.StateType, define.ErrInvalidArg)
 	}
 
 	// Grab config from the database so we can reset some defaults
 	dbConfig, err := runtime.state.GetDBConfig()
 	if err != nil {
-		return errors.Wrapf(err, "error retrieving runtime configuration from database")
+		if runtime.doReset {
+			// We can at least delete the DB and the static files
+			// directory.
+			// Can't safely touch anything else because we aren't
+			// sure of other directories.
+			if err := runtime.state.Close(); err != nil {
+				logrus.Errorf("Closing database connection: %v", err)
+			} else {
+				if err := os.RemoveAll(runtime.config.Engine.StaticDir); err != nil {
+					logrus.Errorf("Removing static files directory %v: %v", runtime.config.Engine.StaticDir, err)
+				}
+			}
+		}
+
+		return fmt.Errorf("retrieving runtime configuration from database: %w", err)
 	}
 
 	runtime.mergeDBConfig(dbConfig)
@@ -337,14 +373,14 @@ func makeRuntime(ctx context.Context, runtime *Runtime) (retErr error) {
 		// it will try to use existing XDG_RUNTIME_DIR
 		// if current user has no write access to XDG_RUNTIME_DIR we will fail later
 		if err := unix.Access(runtime.storageConfig.RunRoot, unix.W_OK); err != nil {
-			msg := "XDG_RUNTIME_DIR is pointing to a path which is not writable. Most likely podman will fail."
+			msg := fmt.Sprintf("RunRoot is pointing to a path (%s) which is not writable. Most likely podman will fail.", runtime.storageConfig.RunRoot)
 			if errors.Is(err, os.ErrNotExist) {
 				// if dir does not exists try to create it
 				if err := os.MkdirAll(runtime.storageConfig.RunRoot, 0700); err != nil {
 					logrus.Warn(msg)
 				}
 			} else {
-				logrus.Warn(msg)
+				logrus.Warnf("%s: %v", msg, err)
 			}
 		}
 	}
@@ -359,11 +395,17 @@ func makeRuntime(ctx context.Context, runtime *Runtime) (retErr error) {
 	// Validate our config against the database, now that we've set our
 	// final storage configuration
 	if err := runtime.state.ValidateDBConfig(runtime); err != nil {
-		return err
+		// If we are performing a storage reset: continue on with a
+		// warning. Otherwise we can't `system reset` after a change to
+		// the core paths.
+		if !runtime.doReset {
+			return err
+		}
+		logrus.Errorf("Runtime paths differ from those stored in database, storage reset may not remove all files")
 	}
 
 	if err := runtime.state.SetNamespace(runtime.config.Engine.Namespace); err != nil {
-		return errors.Wrapf(err, "error setting libpod namespace in state")
+		return fmt.Errorf("setting libpod namespace in state: %w", err)
 	}
 	logrus.Debugf("Set libpod namespace to %q", runtime.config.Engine.Namespace)
 
@@ -381,6 +423,14 @@ func makeRuntime(ctx context.Context, runtime *Runtime) (retErr error) {
 	} else if runtime.noStore {
 		logrus.Debug("No store required. Not opening container store.")
 	} else if err := runtime.configureStore(); err != nil {
+		// Make a best-effort attempt to clean up if performing a
+		// storage reset.
+		if runtime.doReset {
+			if err := runtime.removeAllDirs(); err != nil {
+				logrus.Errorf("Removing libpod directories: %v", err)
+			}
+		}
+
 		return err
 	}
 	defer func() {
@@ -388,18 +438,17 @@ func makeRuntime(ctx context.Context, runtime *Runtime) (retErr error) {
 			// Don't forcibly shut down
 			// We could be opening a store in use by another libpod
 			if _, err := store.Shutdown(false); err != nil {
-				logrus.Errorf("Error removing store for partially-created runtime: %s", err)
+				logrus.Errorf("Removing store for partially-created runtime: %s", err)
 			}
 		}
 	}()
 
-	// Setup the eventer
+	// Set up the eventer
 	eventer, err := runtime.newEventer()
 	if err != nil {
 		return err
 	}
 	runtime.eventer = eventer
-	// TODO: events for libimage
 
 	// Set up containers/image
 	if runtime.imageContext == nil {
@@ -412,16 +461,8 @@ func makeRuntime(ctx context.Context, runtime *Runtime) (retErr error) {
 	// Create the tmpDir
 	if err := os.MkdirAll(runtime.config.Engine.TmpDir, 0751); err != nil {
 		// The directory is allowed to exist
-		if !os.IsExist(err) {
-			return errors.Wrap(err, "error creating tmpdir")
-		}
-	}
-
-	// Create events log dir
-	if err := os.MkdirAll(filepath.Dir(runtime.config.Engine.EventsLogFilePath), 0700); err != nil {
-		// The directory is allowed to exist
-		if !os.IsExist(err) {
-			return errors.Wrap(err, "error creating events dirs")
+		if !errors.Is(err, os.ErrExist) {
+			return fmt.Errorf("creating tmpdir: %w", err)
 		}
 	}
 
@@ -436,7 +477,7 @@ func makeRuntime(ctx context.Context, runtime *Runtime) (retErr error) {
 			// This will allow us to ship configs including optional
 			// runtimes that might not be installed (crun, kata).
 			// Only a infof so default configs don't spec errors.
-			logrus.Debugf("configured OCI runtime %s initialization failed: %v", name, err)
+			logrus.Debugf("Configured OCI runtime %s initialization failed: %v", name, err)
 			continue
 		}
 
@@ -458,7 +499,7 @@ func makeRuntime(ctx context.Context, runtime *Runtime) (retErr error) {
 		} else {
 			ociRuntime, ok := runtime.ociRuntimes[runtime.config.Engine.OCIRuntime]
 			if !ok {
-				return errors.Wrapf(define.ErrInvalidArg, "default OCI runtime %q not found", runtime.config.Engine.OCIRuntime)
+				return fmt.Errorf("default OCI runtime %q not found: %w", runtime.config.Engine.OCIRuntime, define.ErrInvalidArg)
 			}
 			runtime.defaultOCIRuntime = ociRuntime
 		}
@@ -467,34 +508,32 @@ func makeRuntime(ctx context.Context, runtime *Runtime) (retErr error) {
 
 	// Do we have at least one valid OCI runtime?
 	if len(runtime.ociRuntimes) == 0 {
-		return errors.Wrapf(define.ErrInvalidArg, "no OCI runtime has been configured")
+		return fmt.Errorf("no OCI runtime has been configured: %w", define.ErrInvalidArg)
 	}
 
 	// Do we have a default runtime?
 	if runtime.defaultOCIRuntime == nil {
-		return errors.Wrapf(define.ErrInvalidArg, "no default OCI runtime was configured")
+		return fmt.Errorf("no default OCI runtime was configured: %w", define.ErrInvalidArg)
 	}
 
 	// Make the per-boot files directory if it does not exist
 	if err := os.MkdirAll(runtime.config.Engine.TmpDir, 0755); err != nil {
 		// The directory is allowed to exist
-		if !os.IsExist(err) {
-			return errors.Wrapf(err, "error creating runtime temporary files directory")
+		if !errors.Is(err, os.ErrExist) {
+			return fmt.Errorf("creating runtime temporary files directory: %w", err)
 		}
 	}
 
-	// If we need to make a default network - do so now.
-	if err := defaultnet.Create(runtime.config.Network.DefaultNetwork, runtime.config.Network.DefaultSubnet, runtime.config.Network.NetworkConfigDir, runtime.config.Engine.StaticDir, runtime.config.Engine.MachineEnabled); err != nil {
-		logrus.Errorf("Failed to created default CNI network: %v", err)
+	// the store is only set up when we are in the userns so we do the same for the network interface
+	if !needsUserns {
+		netBackend, netInterface, err := network.NetworkBackend(runtime.store, runtime.config, runtime.syslog)
+		if err != nil {
+			return err
+		}
+		runtime.config.Network.NetworkBackend = string(netBackend)
+		runtime.network = netInterface
 	}
 
-	// Set up the CNI net plugin
-	netPlugin, err := ocicni.InitCNINoInotify(runtime.config.Network.DefaultNetwork, runtime.config.Network.NetworkConfigDir, "", runtime.config.Network.CNIPluginDirs...)
-	if err != nil {
-		return errors.Wrapf(err, "error configuring CNI network plugin")
-	}
-	runtime.netPlugin = netPlugin
-
 	// We now need to see if the system has restarted
 	// We check for the presence of a file in our tmp directory to verify this
 	// This check must be locked to prevent races
@@ -502,12 +541,10 @@ func makeRuntime(ctx context.Context, runtime *Runtime) (retErr error) {
 	runtimeAliveFile := filepath.Join(runtime.config.Engine.TmpDir, "alive")
 	aliveLock, err := storage.GetLockfile(runtimeAliveLock)
 	if err != nil {
-		return errors.Wrapf(err, "error acquiring runtime init lock")
+		return fmt.Errorf("acquiring runtime init lock: %w", err)
 	}
 	// Acquire the lock and hold it until we return
 	// This ensures that no two processes will be in runtime.refresh at once
-	// TODO: we can't close the FD in this lock, so we should keep it around
-	// and use it to lock important operations
 	aliveLock.Lock()
 	doRefresh := false
 	defer func() {
@@ -534,7 +571,7 @@ func makeRuntime(ctx context.Context, runtime *Runtime) (retErr error) {
 			aliveLock.Unlock() // Unlock to avoid deadlock as BecomeRootInUserNS will reexec.
 			pausePid, err := util.GetRootlessPauseProcessPidPathGivenDir(runtime.config.Engine.TmpDir)
 			if err != nil {
-				return errors.Wrapf(err, "could not get pause process pid file path")
+				return fmt.Errorf("could not get pause process pid file path: %w", err)
 			}
 			became, ret, err := rootless.BecomeRootInUserNS(pausePid)
 			if err != nil {
@@ -544,6 +581,10 @@ func makeRuntime(ctx context.Context, runtime *Runtime) (retErr error) {
 				// Check if the pause process was created.  If it was created, then
 				// move it to its own systemd scope.
 				utils.MovePauseProcessToScope(pausePid)
+
+				// gocritic complains because defer is not run on os.Exit()
+				// However this is fine because the lock is released anyway when the process exits
+				//nolint:gocritic
 				os.Exit(ret)
 			}
 		}
@@ -551,10 +592,10 @@ func makeRuntime(ctx context.Context, runtime *Runtime) (retErr error) {
 		// This will trigger on first use as well, but refreshing an
 		// empty state only creates a single file
 		// As such, it's not really a performance concern
-		if os.IsNotExist(err) {
+		if errors.Is(err, os.ErrNotExist) {
 			doRefresh = true
 		} else {
-			return errors.Wrapf(err, "error reading runtime status file %s", runtimeAliveFile)
+			return fmt.Errorf("reading runtime status file %s: %w", runtimeAliveFile, err)
 		}
 	}
 
@@ -563,6 +604,18 @@ func makeRuntime(ctx context.Context, runtime *Runtime) (retErr error) {
 		return err
 	}
 
+	// If we're resetting storage, do it now.
+	// We will not return a valid runtime.
+	// TODO: Plumb this context out so it can be set.
+	if runtime.doReset {
+		// Mark the runtime as valid, so normal functionality "mostly"
+		// works and we can use regular functions to remove
+		// ctrs/pods/etc
+		runtime.valid = true
+
+		return runtime.reset(context.Background())
+	}
+
 	// If we're renumbering locks, do it now.
 	// It breaks out of normal runtime init, and will not return a valid
 	// runtime.
@@ -587,12 +640,14 @@ func makeRuntime(ctx context.Context, runtime *Runtime) (retErr error) {
 		}
 	}
 
+	runtime.startWorker()
+
 	// Mark the runtime as valid - ready to be used, cannot be modified
 	// further
 	runtime.valid = true
 
 	if runtime.doMigrate {
-		if err := runtime.migrate(ctx); err != nil {
+		if err := runtime.migrate(); err != nil {
 			return err
 		}
 	}
@@ -600,102 +655,6 @@ func makeRuntime(ctx context.Context, runtime *Runtime) (retErr error) {
 	return nil
 }
 
-// findConmon iterates over conmonPaths and returns the path
-// to the first conmon binary with a new enough version. If none is found,
-// we try to do a path lookup of "conmon".
-func findConmon(conmonPaths []string) (string, error) {
-	foundOutdatedConmon := false
-	for _, path := range conmonPaths {
-		stat, err := os.Stat(path)
-		if err != nil {
-			continue
-		}
-		if stat.IsDir() {
-			continue
-		}
-		if err := probeConmon(path); err != nil {
-			logrus.Warnf("Conmon at %s invalid: %v", path, err)
-			foundOutdatedConmon = true
-			continue
-		}
-		logrus.Debugf("Using conmon: %q", path)
-		return path, nil
-	}
-
-	// Search the $PATH as last fallback
-	if path, err := exec.LookPath("conmon"); err == nil {
-		if err := probeConmon(path); err != nil {
-			logrus.Warnf("Conmon at %s is invalid: %v", path, err)
-			foundOutdatedConmon = true
-		} else {
-			logrus.Debugf("Using conmon from $PATH: %q", path)
-			return path, nil
-		}
-	}
-
-	if foundOutdatedConmon {
-		return "", errors.Wrapf(define.ErrConmonOutdated,
-			"please update to v%d.%d.%d or later",
-			conmonMinMajorVersion, conmonMinMinorVersion, conmonMinPatchVersion)
-	}
-
-	return "", errors.Wrapf(define.ErrInvalidArg,
-		"could not find a working conmon binary (configured options: %v)",
-		conmonPaths)
-}
-
-// probeConmon calls conmon --version and verifies it is a new enough version for
-// the runtime expectations the container engine currently has.
-func probeConmon(conmonBinary string) error {
-	cmd := exec.Command(conmonBinary, "--version")
-	var out bytes.Buffer
-	cmd.Stdout = &out
-	err := cmd.Run()
-	if err != nil {
-		return err
-	}
-	r := regexp.MustCompile(`^conmon version (?P<Major>\d+).(?P<Minor>\d+).(?P<Patch>\d+)`)
-
-	matches := r.FindStringSubmatch(out.String())
-	if len(matches) != 4 {
-		return errors.Wrap(err, define.ErrConmonVersionFormat)
-	}
-	major, err := strconv.Atoi(matches[1])
-	if err != nil {
-		return errors.Wrap(err, define.ErrConmonVersionFormat)
-	}
-	if major < conmonMinMajorVersion {
-		return define.ErrConmonOutdated
-	}
-	if major > conmonMinMajorVersion {
-		return nil
-	}
-
-	minor, err := strconv.Atoi(matches[2])
-	if err != nil {
-		return errors.Wrap(err, define.ErrConmonVersionFormat)
-	}
-	if minor < conmonMinMinorVersion {
-		return define.ErrConmonOutdated
-	}
-	if minor > conmonMinMinorVersion {
-		return nil
-	}
-
-	patch, err := strconv.Atoi(matches[3])
-	if err != nil {
-		return errors.Wrap(err, define.ErrConmonVersionFormat)
-	}
-	if patch < conmonMinPatchVersion {
-		return define.ErrConmonOutdated
-	}
-	if patch > conmonMinPatchVersion {
-		return nil
-	}
-
-	return nil
-}
-
 // TmpDir gets the current Libpod temporary files directory.
 func (r *Runtime) TmpDir() (string, error) {
 	if !r.valid {
@@ -705,20 +664,30 @@ func (r *Runtime) TmpDir() (string, error) {
 	return r.config.Engine.TmpDir, nil
 }
 
-// GetConfig returns a copy of the configuration used by the runtime
-func (r *Runtime) GetConfig() (*config.Config, error) {
-	r.lock.RLock()
-	defer r.lock.RUnlock()
-
+// GetConfig returns the configuration used by the runtime.
+// Note that the returned value is not a copy and must hence
+// only be used in a reading fashion.
+func (r *Runtime) GetConfigNoCopy() (*config.Config, error) {
 	if !r.valid {
 		return nil, define.ErrRuntimeStopped
 	}
+	return r.config, nil
+}
+
+// GetConfig returns a copy of the configuration used by the runtime.
+// Please use GetConfigNoCopy() in case you only want to read from
+// but not write to the returned config.
+func (r *Runtime) GetConfig() (*config.Config, error) {
+	rtConfig, err := r.GetConfigNoCopy()
+	if err != nil {
+		return nil, err
+	}
 
 	config := new(config.Config)
 
 	// Copy so the caller won't be able to modify the actual config
-	if err := JSONDeepCopy(r.config, config); err != nil {
-		return nil, errors.Wrapf(err, "error copying config")
+	if err := JSONDeepCopy(rtConfig, config); err != nil {
+		return nil, fmt.Errorf("copying config: %w", err)
 	}
 
 	return config, nil
@@ -753,9 +722,10 @@ func (r *Runtime) libimageEvents() {
 
 	eventChannel := r.libimageRuntime.EventChannel()
 	go func() {
+		sawShutdown := false
 		for {
 			// Make sure to read and write all events before
-			// checking if we're about to shutdown.
+			// shutting down.
 			for len(eventChannel) > 0 {
 				libimageEvent := <-eventChannel
 				e := events.Event{
@@ -766,16 +736,19 @@ func (r *Runtime) libimageEvents() {
 					Type:   events.Image,
 				}
 				if err := r.eventer.Write(e); err != nil {
-					logrus.Errorf("unable to write image event: %q", err)
+					logrus.Errorf("Unable to write image event: %q", err)
 				}
 			}
 
+			if sawShutdown {
+				close(r.libimageEventsShutdown)
+				return
+			}
+
 			select {
 			case <-r.libimageEventsShutdown:
-				return
-
-			default:
-				time.Sleep(100 * time.Millisecond)
+				sawShutdown = true
+			case <-time.After(100 * time.Millisecond):
 			}
 		}
 	}()
@@ -793,11 +766,13 @@ func (r *Runtime) DeferredShutdown(force bool) {
 // cleaning up; if force is false, an error will be returned if there are
 // still containers running or mounted
 func (r *Runtime) Shutdown(force bool) error {
-	r.lock.Lock()
-	defer r.lock.Unlock()
-
 	if !r.valid {
-		return define.ErrRuntimeStopped
+		return nil
+	}
+
+	if r.workerChannel != nil {
+		r.workerGroup.Wait()
+		close(r.workerChannel)
 	}
 
 	r.valid = false
@@ -806,11 +781,11 @@ func (r *Runtime) Shutdown(force bool) error {
 	if force {
 		ctrs, err := r.state.AllContainers()
 		if err != nil {
-			logrus.Errorf("Error retrieving containers from database: %v", err)
+			logrus.Errorf("Retrieving containers from database: %v", err)
 		} else {
 			for _, ctr := range ctrs {
 				if err := ctr.StopWithTimeout(r.config.Engine.StopTimeout); err != nil {
-					logrus.Errorf("Error stopping container %s: %v", ctr.ID(), err)
+					logrus.Errorf("Stopping container %s: %v", ctr.ID(), err)
 				}
 			}
 		}
@@ -822,17 +797,20 @@ func (r *Runtime) Shutdown(force bool) error {
 	if r.store != nil {
 		// Wait for the events to be written.
 		if r.libimageEventsShutdown != nil {
+			// Tell loop to shutdown
 			r.libimageEventsShutdown <- true
+			// Wait for close to signal shutdown
+			<-r.libimageEventsShutdown
 		}
 
 		// Note that the libimage runtime shuts down the store.
 		if err := r.libimageRuntime.Shutdown(force); err != nil {
-			lastError = errors.Wrapf(err, "error shutting down container storage")
+			lastError = fmt.Errorf("shutting down container storage: %w", err)
 		}
 	}
 	if err := r.state.Close(); err != nil {
 		if lastError != nil {
-			logrus.Errorf("%v", lastError)
+			logrus.Error(lastError)
 		}
 		lastError = err
 	}
@@ -859,15 +837,15 @@ func (r *Runtime) refresh(alivePath string) error {
 	// Containers, pods, and volumes must also reacquire their locks.
 	ctrs, err := r.state.AllContainers()
 	if err != nil {
-		return errors.Wrapf(err, "error retrieving all containers from state")
+		return fmt.Errorf("retrieving all containers from state: %w", err)
 	}
 	pods, err := r.state.AllPods()
 	if err != nil {
-		return errors.Wrapf(err, "error retrieving all pods from state")
+		return fmt.Errorf("retrieving all pods from state: %w", err)
 	}
 	vols, err := r.state.AllVolumes()
 	if err != nil {
-		return errors.Wrapf(err, "error retrieving all volumes from state")
+		return fmt.Errorf("retrieving all volumes from state: %w", err)
 	}
 	// No locks are taken during pod, volume, and container refresh.
 	// Furthermore, the pod/volume/container refresh() functions are not
@@ -878,28 +856,28 @@ func (r *Runtime) refresh(alivePath string) error {
 	// until this has run.
 	for _, ctr := range ctrs {
 		if err := ctr.refresh(); err != nil {
-			logrus.Errorf("Error refreshing container %s: %v", ctr.ID(), err)
+			logrus.Errorf("Refreshing container %s: %v", ctr.ID(), err)
 		}
 	}
 	for _, pod := range pods {
 		if err := pod.refresh(); err != nil {
-			logrus.Errorf("Error refreshing pod %s: %v", pod.ID(), err)
+			logrus.Errorf("Refreshing pod %s: %v", pod.ID(), err)
 		}
 	}
 	for _, vol := range vols {
 		if err := vol.refresh(); err != nil {
-			logrus.Errorf("Error refreshing volume %s: %v", vol.Name(), err)
+			logrus.Errorf("Refreshing volume %s: %v", vol.Name(), err)
 		}
 	}
 
 	// Create a file indicating the runtime is alive and ready
 	file, err := os.OpenFile(alivePath, os.O_RDONLY|os.O_CREATE, 0644)
 	if err != nil {
-		return errors.Wrap(err, "error creating runtime status file")
+		return fmt.Errorf("creating runtime status file: %w", err)
 	}
 	defer file.Close()
 
-	r.newSystemEvent(events.Refresh)
+	r.NewSystemEvent(events.Refresh)
 
 	return nil
 }
@@ -916,13 +894,13 @@ func (r *Runtime) generateName() (string, error) {
 		// Make sure container with this name does not exist
 		if _, err := r.state.LookupContainer(name); err == nil {
 			continue
-		} else if errors.Cause(err) != define.ErrNoSuchCtr {
+		} else if !errors.Is(err, define.ErrNoSuchCtr) {
 			return "", err
 		}
 		// Make sure pod with this name does not exist
 		if _, err := r.state.LookupPod(name); err == nil {
 			continue
-		} else if errors.Cause(err) != define.ErrNoSuchPod {
+		} else if !errors.Is(err, define.ErrNoSuchPod) {
 			return "", err
 		}
 		return name, nil
@@ -999,9 +977,6 @@ func (r *Runtime) RunRoot() string {
 // If the given ID does not correspond to any existing Pod or Container,
 // ErrNoSuchCtr is returned.
 func (r *Runtime) GetName(id string) (string, error) {
-	r.lock.RLock()
-	defer r.lock.RUnlock()
-
 	if !r.valid {
 		return "", define.ErrRuntimeStopped
 	}
@@ -1044,7 +1019,7 @@ func (r *Runtime) mergeDBConfig(dbConfig *DBConfig) {
 	if !r.storageSet.GraphDriverNameSet && dbConfig.GraphDriver != "" {
 		if r.storageConfig.GraphDriverName != dbConfig.GraphDriver &&
 			r.storageConfig.GraphDriverName != "" {
-			logrus.Errorf("User-selected graph driver %q overwritten by graph driver %q from database - delete libpod local files to resolve",
+			logrus.Errorf("User-selected graph driver %q overwritten by graph driver %q from database - delete libpod local files to resolve.  May prevent use of images created by other tools",
 				r.storageConfig.GraphDriverName, dbConfig.GraphDriver)
 		}
 		r.storageConfig.GraphDriverName = dbConfig.GraphDriver
@@ -1062,7 +1037,6 @@ func (r *Runtime) mergeDBConfig(dbConfig *DBConfig) {
 			logrus.Debugf("Overriding tmp dir %q with %q from database", c.TmpDir, dbConfig.LibpodTmp)
 		}
 		c.TmpDir = dbConfig.LibpodTmp
-		c.EventsLogFilePath = filepath.Join(dbConfig.LibpodTmp, "events", "events.log")
 	}
 
 	if !r.storageSet.VolumePathSet && dbConfig.VolumePath != "" {
@@ -1098,7 +1072,7 @@ func (r *Runtime) reloadContainersConf() error {
 		return err
 	}
 	r.config = config
-	logrus.Infof("applied new containers configuration: %v", config)
+	logrus.Infof("Applied new containers configuration: %v", config)
 	return nil
 }
 
@@ -1109,26 +1083,31 @@ func (r *Runtime) reloadStorageConf() error {
 		return err
 	}
 	storage.ReloadConfigurationFile(configFile, &r.storageConfig)
-	logrus.Infof("applied new storage configuration: %v", r.storageConfig)
+	logrus.Infof("Applied new storage configuration: %v", r.storageConfig)
 	return nil
 }
 
-// getVolumePlugin gets a specific volume plugin given its name.
-func (r *Runtime) getVolumePlugin(name string) (*plugin.VolumePlugin, error) {
+// getVolumePlugin gets a specific volume plugin.
+func (r *Runtime) getVolumePlugin(volConfig *VolumeConfig) (*plugin.VolumePlugin, error) {
 	// There is no plugin for local.
+	name := volConfig.Driver
+	timeout := volConfig.Timeout
 	if name == define.VolumeDriverLocal || name == "" {
 		return nil, nil
 	}
 
 	pluginPath, ok := r.config.Engine.VolumePlugins[name]
 	if !ok {
-		return nil, errors.Wrapf(define.ErrMissingPlugin, "no volume plugin with name %s available", name)
+		if name == define.VolumeDriverImage {
+			return nil, nil
+		}
+		return nil, fmt.Errorf("no volume plugin with name %s available: %w", name, define.ErrMissingPlugin)
 	}
 
-	return plugin.GetVolumePlugin(name, pluginPath)
+	return plugin.GetVolumePlugin(name, pluginPath, timeout, r.config)
 }
 
-// GetSecretsStoreageDir returns the directory that the secrets manager should take
+// GetSecretsStorageDir returns the directory that the secrets manager should take
 func (r *Runtime) GetSecretsStorageDir() string {
 	return filepath.Join(r.store.GraphRoot(), "secrets")
 }
@@ -1170,3 +1149,23 @@ func (r *Runtime) graphRootMountedFlag(mounts []spec.Mount) string {
 	}
 	return ""
 }
+
+// Network returns the network interface which is used by the runtime
+func (r *Runtime) Network() nettypes.ContainerNetwork {
+	return r.network
+}
+
+// GetDefaultNetworkName returns the network interface which is used by the runtime
+func (r *Runtime) GetDefaultNetworkName() string {
+	return r.config.Network.DefaultNetwork
+}
+
+// RemoteURI returns the API server URI
+func (r *Runtime) RemoteURI() string {
+	return r.config.Engine.RemoteURI
+}
+
+// SetRemoteURI records the API server URI
+func (r *Runtime) SetRemoteURI(uri string) {
+	r.config.Engine.RemoteURI = uri
+}
