@@ -2,6 +2,7 @@ package compat
 
 import (
 	"encoding/json"
+	"errors"
 	"fmt"
 	"net/http"
 	"sort"
@@ -10,42 +11,42 @@ import (
 	"syscall"
 	"time"
 
-	"github.com/containers/podman/v3/libpod"
-	"github.com/containers/podman/v3/libpod/define"
-	"github.com/containers/podman/v3/pkg/api/handlers"
-	"github.com/containers/podman/v3/pkg/api/handlers/utils"
-	api "github.com/containers/podman/v3/pkg/api/types"
-	"github.com/containers/podman/v3/pkg/domain/entities"
-	"github.com/containers/podman/v3/pkg/domain/filters"
-	"github.com/containers/podman/v3/pkg/domain/infra/abi"
-	"github.com/containers/podman/v3/pkg/ps"
-	"github.com/containers/podman/v3/pkg/signal"
-	"github.com/containers/podman/v3/pkg/util"
+	"github.com/containers/podman/v4/libpod"
+	"github.com/containers/podman/v4/libpod/define"
+	"github.com/containers/podman/v4/pkg/api/handlers"
+	"github.com/containers/podman/v4/pkg/api/handlers/utils"
+	api "github.com/containers/podman/v4/pkg/api/types"
+	"github.com/containers/podman/v4/pkg/domain/entities"
+	"github.com/containers/podman/v4/pkg/domain/filters"
+	"github.com/containers/podman/v4/pkg/domain/infra/abi"
+	"github.com/containers/podman/v4/pkg/ps"
+	"github.com/containers/podman/v4/pkg/signal"
+	"github.com/containers/podman/v4/pkg/util"
 	"github.com/docker/docker/api/types"
 	"github.com/docker/docker/api/types/container"
 	"github.com/docker/docker/api/types/network"
 	"github.com/docker/go-connections/nat"
 	"github.com/docker/go-units"
 	"github.com/gorilla/schema"
-	"github.com/pkg/errors"
 	"github.com/sirupsen/logrus"
 )
 
 func RemoveContainer(w http.ResponseWriter, r *http.Request) {
 	decoder := r.Context().Value(api.DecoderKey).(*schema.Decoder)
 	query := struct {
-		Force         bool `schema:"force"`
-		Ignore        bool `schema:"ignore"`
-		Link          bool `schema:"link"`
-		DockerVolumes bool `schema:"v"`
-		LibpodVolumes bool `schema:"volumes"`
+		Force         bool  `schema:"force"`
+		Ignore        bool  `schema:"ignore"`
+		Depend        bool  `schema:"depend"`
+		Link          bool  `schema:"link"`
+		Timeout       *uint `schema:"timeout"`
+		DockerVolumes bool  `schema:"v"`
+		LibpodVolumes bool  `schema:"volumes"`
 	}{
 		// override any golang type defaults
 	}
 
 	if err := decoder.Decode(&query, r.URL.Query()); err != nil {
-		utils.Error(w, http.StatusText(http.StatusBadRequest), http.StatusBadRequest,
-			errors.Wrapf(err, "failed to parse parameters for %s", r.URL.String()))
+		utils.Error(w, http.StatusBadRequest, fmt.Errorf("failed to parse parameters for %s: %w", r.URL.String(), err))
 		return
 	}
 
@@ -55,10 +56,11 @@ func RemoveContainer(w http.ResponseWriter, r *http.Request) {
 	}
 	if utils.IsLibpodRequest(r) {
 		options.Volumes = query.LibpodVolumes
+		options.Timeout = query.Timeout
+		options.Depend = query.Depend
 	} else {
 		if query.Link {
-			utils.Error(w, http.StatusText(http.StatusBadRequest), http.StatusBadRequest,
-				utils.ErrLinkNotSupport)
+			utils.Error(w, http.StatusBadRequest, utils.ErrLinkNotSupport)
 			return
 		}
 		options.Volumes = query.DockerVolumes
@@ -69,9 +71,9 @@ func RemoveContainer(w http.ResponseWriter, r *http.Request) {
 	// code.
 	containerEngine := abi.ContainerEngine{Libpod: runtime}
 	name := utils.GetName(r)
-	report, err := containerEngine.ContainerRm(r.Context(), []string{name}, options)
+	reports, err := containerEngine.ContainerRm(r.Context(), []string{name}, options)
 	if err != nil {
-		if errors.Cause(err) == define.ErrNoSuchCtr {
+		if errors.Is(err, define.ErrNoSuchCtr) {
 			utils.ContainerNotFound(w, name, err)
 			return
 		}
@@ -79,16 +81,19 @@ func RemoveContainer(w http.ResponseWriter, r *http.Request) {
 		utils.InternalServerError(w, err)
 		return
 	}
-	if len(report) > 0 && report[0].Err != nil {
-		err = report[0].Err
-		if errors.Cause(err) == define.ErrNoSuchCtr {
+	if len(reports) > 0 && reports[0].Err != nil {
+		err = reports[0].Err
+		if errors.Is(err, define.ErrNoSuchCtr) {
 			utils.ContainerNotFound(w, name, err)
 			return
 		}
 		utils.InternalServerError(w, err)
 		return
 	}
-
+	if utils.IsLibpodRequest(r) {
+		utils.WriteResponse(w, http.StatusOK, reports)
+		return
+	}
 	utils.WriteResponse(w, http.StatusNoContent, nil)
 }
 
@@ -105,12 +110,12 @@ func ListContainers(w http.ResponseWriter, r *http.Request) {
 
 	filterMap, err := util.PrepareFilters(r)
 	if err != nil {
-		utils.Error(w, "Something went wrong.", http.StatusInternalServerError, errors.Wrapf(err, "failed to decode filter parameters for %s", r.URL.String()))
+		utils.Error(w, http.StatusInternalServerError, fmt.Errorf("failed to decode filter parameters for %s: %w", r.URL.String(), err))
 		return
 	}
 
 	if err := decoder.Decode(&query, r.URL.Query()); err != nil {
-		utils.Error(w, "Something went wrong.", http.StatusInternalServerError, errors.Wrapf(err, "failed to parse parameters for %s", r.URL.String()))
+		utils.Error(w, http.StatusInternalServerError, fmt.Errorf("failed to parse parameters for %s: %w", r.URL.String(), err))
 		return
 	}
 
@@ -159,7 +164,7 @@ func ListContainers(w http.ResponseWriter, r *http.Request) {
 	for _, ctnr := range containers {
 		api, err := LibpodToContainer(ctnr, query.Size)
 		if err != nil {
-			if errors.Cause(err) == define.ErrNoSuchCtr {
+			if errors.Is(err, define.ErrNoSuchCtr) {
 				// container was removed between the initial fetch of the list and conversion
 				logrus.Debugf("Container %s removed between initial fetch and conversion, ignoring in output", ctnr.ID())
 				continue
@@ -182,7 +187,7 @@ func GetContainer(w http.ResponseWriter, r *http.Request) {
 	}
 
 	if err := decoder.Decode(&query, r.URL.Query()); err != nil {
-		utils.Error(w, "Something went wrong.", http.StatusBadRequest, errors.Wrapf(err, "failed to parse parameters for %s", r.URL.String()))
+		utils.Error(w, http.StatusBadRequest, fmt.Errorf("failed to parse parameters for %s: %w", r.URL.String(), err))
 		return
 	}
 
@@ -210,7 +215,7 @@ func KillContainer(w http.ResponseWriter, r *http.Request) {
 		Signal: "KILL",
 	}
 	if err := decoder.Decode(&query, r.URL.Query()); err != nil {
-		utils.Error(w, "Something went wrong.", http.StatusBadRequest, errors.Wrapf(err, "failed to parse parameters for %s", r.URL.String()))
+		utils.Error(w, http.StatusBadRequest, fmt.Errorf("failed to parse parameters for %s: %w", r.URL.String(), err))
 		return
 	}
 
@@ -223,12 +228,12 @@ func KillContainer(w http.ResponseWriter, r *http.Request) {
 	}
 	report, err := containerEngine.ContainerKill(r.Context(), []string{name}, options)
 	if err != nil {
-		if errors.Cause(err) == define.ErrCtrStateInvalid ||
-			errors.Cause(err) == define.ErrCtrStopped {
-			utils.Error(w, fmt.Sprintf("Container %s is not running", name), http.StatusConflict, err)
+		if errors.Is(err, define.ErrCtrStateInvalid) ||
+			errors.Is(err, define.ErrCtrStopped) {
+			utils.Error(w, http.StatusConflict, err)
 			return
 		}
-		if errors.Cause(err) == define.ErrNoSuchCtr {
+		if errors.Is(err, define.ErrNoSuchCtr) {
 			utils.ContainerNotFound(w, name, err)
 			return
 		}
@@ -249,13 +254,13 @@ func KillContainer(w http.ResponseWriter, r *http.Request) {
 			utils.InternalServerError(w, err)
 			return
 		}
-		if sig == 0 || syscall.Signal(sig) == syscall.SIGKILL {
+		if sig == 0 || sig == syscall.SIGKILL {
 			opts := entities.WaitOptions{
 				Condition: []define.ContainerStatus{define.ContainerStateExited, define.ContainerStateStopped},
 				Interval:  time.Millisecond * 250,
 			}
 			if _, err := containerEngine.ContainerWait(r.Context(), []string{name}, opts); err != nil {
-				utils.Error(w, "Something went wrong.", http.StatusInternalServerError, err)
+				utils.Error(w, http.StatusInternalServerError, err)
 				return
 			}
 		}
@@ -284,13 +289,16 @@ func LibpodToContainer(l *libpod.Container, sz bool) (*handlers.Container, error
 		return nil, err
 	}
 	stateStr := state.String()
-	if stateStr == "configured" {
-		stateStr = "created"
+
+	// Some docker states are not the same as ours. This makes sure the state string stays true to the Docker API
+	if state == define.ContainerStateCreated {
+		stateStr = define.ContainerStateConfigured.String()
 	}
 
-	if state == define.ContainerStateConfigured || state == define.ContainerStateCreated {
+	switch state {
+	case define.ContainerStateConfigured, define.ContainerStateCreated:
 		status = "Created"
-	} else if state == define.ContainerStateStopped || state == define.ContainerStateExited {
+	case define.ContainerStateStopped, define.ContainerStateExited:
 		exitCode, _, err := l.ExitCode()
 		if err != nil {
 			return nil, err
@@ -300,7 +308,7 @@ func LibpodToContainer(l *libpod.Container, sz bool) (*handlers.Container, error
 			return nil, err
 		}
 		status = fmt.Sprintf("Exited (%d) %s ago", exitCode, units.HumanDuration(time.Since(finishedTime)))
-	} else if state == define.ContainerStateRunning || state == define.ContainerStatePaused {
+	case define.ContainerStateRunning, define.ContainerStatePaused:
 		startedTime, err := l.StartedTime()
 		if err != nil {
 			return nil, err
@@ -309,11 +317,11 @@ func LibpodToContainer(l *libpod.Container, sz bool) (*handlers.Container, error
 		if state == define.ContainerStatePaused {
 			status += " (Paused)"
 		}
-	} else if state == define.ContainerStateRemoving {
+	case define.ContainerStateRemoving:
 		status = "Removal In Progress"
-	} else if state == define.ContainerStateStopping {
+	case define.ContainerStateStopping:
 		status = "Stopping"
-	} else {
+	default:
 		status = "Unknown"
 	}
 
@@ -335,8 +343,8 @@ func LibpodToContainer(l *libpod.Container, sz bool) (*handlers.Container, error
 	for idx, portMapping := range portMappings {
 		ports[idx] = types.Port{
 			IP:          portMapping.HostIP,
-			PrivatePort: uint16(portMapping.ContainerPort),
-			PublicPort:  uint16(portMapping.HostPort),
+			PrivatePort: portMapping.ContainerPort,
+			PublicPort:  portMapping.HostPort,
 			Type:        portMapping.Protocol,
 		}
 	}
@@ -354,32 +362,52 @@ func LibpodToContainer(l *libpod.Container, sz bool) (*handlers.Container, error
 		return nil, err
 	}
 
-	return &handlers.Container{Container: types.Container{
-		ID:         l.ID(),
-		Names:      []string{fmt.Sprintf("/%s", l.Name())},
-		Image:      imageName,
-		ImageID:    imageID,
-		Command:    strings.Join(l.Command(), " "),
-		Created:    l.CreatedTime().Unix(),
-		Ports:      ports,
-		SizeRw:     sizeRW,
-		SizeRootFs: sizeRootFs,
-		Labels:     l.Labels(),
-		State:      stateStr,
-		Status:     status,
-		HostConfig: struct {
-			NetworkMode string `json:",omitempty"`
-		}{
-			"host"},
-		NetworkSettings: &networkSettings,
-		Mounts:          nil,
-	},
+	m, err := json.Marshal(inspect.Mounts)
+	if err != nil {
+		return nil, err
+	}
+	mounts := []types.MountPoint{}
+	if err := json.Unmarshal(m, &mounts); err != nil {
+		return nil, err
+	}
+
+	return &handlers.Container{
+		Container: types.Container{
+			ID:         l.ID(),
+			Names:      []string{fmt.Sprintf("/%s", l.Name())},
+			Image:      imageName,
+			ImageID:    "sha256:" + imageID,
+			Command:    strings.Join(l.Command(), " "),
+			Created:    l.CreatedTime().Unix(),
+			Ports:      ports,
+			SizeRw:     sizeRW,
+			SizeRootFs: sizeRootFs,
+			Labels:     l.Labels(),
+			State:      stateStr,
+			Status:     status,
+			HostConfig: struct {
+				NetworkMode string `json:",omitempty"`
+			}{
+				"host",
+			},
+			NetworkSettings: &networkSettings,
+			Mounts:          mounts,
+		},
 		ContainerCreateConfig: types.ContainerCreateConfig{},
 	}, nil
 }
 
+func convertSecondaryIPPrefixLen(input *define.InspectNetworkSettings, output *types.NetworkSettings) {
+	for index, ip := range input.SecondaryIPAddresses {
+		output.SecondaryIPAddresses[index].PrefixLen = ip.PrefixLength
+	}
+	for index, ip := range input.SecondaryIPv6Addresses {
+		output.SecondaryIPv6Addresses[index].PrefixLen = ip.PrefixLength
+	}
+}
+
 func LibpodToContainerJSON(l *libpod.Container, sz bool) (*types.ContainerJSON, error) {
-	_, imageName := l.Image()
+	imageID, imageName := l.Image()
 	inspect, err := l.Inspect(sz)
 	if err != nil {
 		return nil, err
@@ -403,18 +431,18 @@ func LibpodToContainerJSON(l *libpod.Container, sz bool) (*types.ContainerJSON,
 		state.Running = true
 	}
 
-	// docker calls the configured state "created"
-	if state.Status == define.ContainerStateConfigured.String() {
-		state.Status = define.ContainerStateCreated.String()
+	// Dockers created state is our configured state
+	if state.Status == define.ContainerStateCreated.String() {
+		state.Status = define.ContainerStateConfigured.String()
 	}
 
 	if l.HasHealthCheck() && state.Status != "created" {
 		state.Health = &types.Health{
-			Status:        inspect.State.Healthcheck.Status,
-			FailingStreak: inspect.State.Healthcheck.FailingStreak,
+			Status:        inspect.State.Health.Status,
+			FailingStreak: inspect.State.Health.FailingStreak,
 		}
 
-		log := inspect.State.Healthcheck.Log
+		log := inspect.State.Health.Log
 
 		for _, item := range log {
 			res := &types.HealthcheckResult{}
@@ -439,6 +467,7 @@ func LibpodToContainerJSON(l *libpod.Container, sz bool) (*types.ContainerJSON,
 	if err := json.Unmarshal(h, &hc); err != nil {
 		return nil, err
 	}
+	sort.Strings(hc.Binds)
 
 	// k8s-file == json-file
 	if hc.LogConfig.Type == define.KubernetesLogging {
@@ -459,7 +488,7 @@ func LibpodToContainerJSON(l *libpod.Container, sz bool) (*types.ContainerJSON,
 		Path:            inspect.Path,
 		Args:            inspect.Args,
 		State:           &state,
-		Image:           imageName,
+		Image:           "sha256:" + imageID,
 		ResolvConfPath:  inspect.ResolvConfPath,
 		HostnamePath:    inspect.HostnamePath,
 		HostsPath:       inspect.HostsPath,
@@ -493,7 +522,7 @@ func LibpodToContainerJSON(l *libpod.Container, sz bool) (*types.ContainerJSON,
 	for ep := range inspect.HostConfig.PortBindings {
 		splitp := strings.SplitN(ep, "/", 2)
 		if len(splitp) != 2 {
-			return nil, errors.Errorf("PORT/PROTOCOL Format required for %q", ep)
+			return nil, fmt.Errorf("PORT/PROTOCOL Format required for %q", ep)
 		}
 		exposedPort, err := nat.NewPort(splitp[1], splitp[0])
 		if err != nil {
@@ -568,6 +597,9 @@ func LibpodToContainerJSON(l *libpod.Container, sz bool) (*types.ContainerJSON,
 	if err := json.Unmarshal(n, &networkSettings); err != nil {
 		return nil, err
 	}
+
+	convertSecondaryIPPrefixLen(inspect.NetworkSettings, &networkSettings)
+
 	// do not report null instead use an empty map
 	if networkSettings.Networks == nil {
 		networkSettings.Networks = map[string]*network.EndpointSettings{}
@@ -597,7 +629,7 @@ func RenameContainer(w http.ResponseWriter, r *http.Request) {
 		Name string `schema:"name"`
 	}{}
 	if err := decoder.Decode(&query, r.URL.Query()); err != nil {
-		utils.Error(w, "Something went wrong.", http.StatusBadRequest, errors.Wrapf(err, "failed to parse parameters for %s", r.URL.String()))
+		utils.Error(w, http.StatusBadRequest, fmt.Errorf("failed to parse parameters for %s: %w", r.URL.String(), err))
 		return
 	}
 
@@ -608,8 +640,8 @@ func RenameContainer(w http.ResponseWriter, r *http.Request) {
 	}
 
 	if _, err := runtime.RenameContainer(r.Context(), ctr, query.Name); err != nil {
-		if errors.Cause(err) == define.ErrPodExists || errors.Cause(err) == define.ErrCtrExists {
-			utils.Error(w, "Something went wrong.", http.StatusConflict, err)
+		if errors.Is(err, define.ErrPodExists) || errors.Is(err, define.ErrCtrExists) {
+			utils.Error(w, http.StatusConflict, err)
 			return
 		}
 		utils.InternalServerError(w, err)
