@@ -3,9 +3,9 @@ package compat
 import (
 	"context"
 	"encoding/json"
+	"errors"
 	"fmt"
 	"io"
-	"io/ioutil"
 	"net/http"
 	"os"
 	"path/filepath"
@@ -16,18 +16,18 @@ import (
 	"github.com/containers/buildah"
 	buildahDefine "github.com/containers/buildah/define"
 	"github.com/containers/buildah/pkg/parse"
-	"github.com/containers/buildah/util"
+	"github.com/containers/image/v5/docker/reference"
 	"github.com/containers/image/v5/types"
-	"github.com/containers/podman/v3/libpod"
-	"github.com/containers/podman/v3/pkg/api/handlers/utils"
-	api "github.com/containers/podman/v3/pkg/api/types"
-	"github.com/containers/podman/v3/pkg/auth"
-	"github.com/containers/podman/v3/pkg/channel"
+	"github.com/containers/podman/v4/libpod"
+	"github.com/containers/podman/v4/pkg/api/handlers/utils"
+	api "github.com/containers/podman/v4/pkg/api/types"
+	"github.com/containers/podman/v4/pkg/auth"
+	"github.com/containers/podman/v4/pkg/channel"
+	"github.com/containers/podman/v4/pkg/rootless"
 	"github.com/containers/storage/pkg/archive"
 	"github.com/docker/docker/pkg/jsonmessage"
 	"github.com/gorilla/schema"
 	"github.com/opencontainers/runtime-spec/specs-go"
-	"github.com/pkg/errors"
 	"github.com/sirupsen/logrus"
 )
 
@@ -65,85 +65,107 @@ func BuildImage(w http.ResponseWriter, r *http.Request) {
 		}
 		err := os.RemoveAll(filepath.Dir(contextDirectory))
 		if err != nil {
-			logrus.Warn(errors.Wrapf(err, "failed to remove build scratch directory %q", filepath.Dir(contextDirectory)))
+			logrus.Warn(fmt.Errorf("failed to remove build scratch directory %q: %w", filepath.Dir(contextDirectory), err))
 		}
 	}()
 
 	query := struct {
-		AddHosts               string   `schema:"extrahosts"`
-		AdditionalCapabilities string   `schema:"addcaps"`
-		Annotations            string   `schema:"annotations"`
-		AppArmor               string   `schema:"apparmor"`
-		BuildArgs              string   `schema:"buildargs"`
-		CacheFrom              string   `schema:"cachefrom"`
-		Compression            uint64   `schema:"compression"`
-		ConfigureNetwork       string   `schema:"networkmode"`
-		CpuPeriod              uint64   `schema:"cpuperiod"`    // nolint
-		CpuQuota               int64    `schema:"cpuquota"`     // nolint
-		CpuSetCpus             string   `schema:"cpusetcpus"`   // nolint
-		CpuSetMems             string   `schema:"cpusetmems"`   // nolint
-		CpuShares              uint64   `schema:"cpushares"`    // nolint
-		CgroupParent           string   `schema:"cgroupparent"` // nolint
-		DNSOptions             string   `schema:"dnsoptions"`
-		DNSSearch              string   `schema:"dnssearch"`
-		DNSServers             string   `schema:"dnsservers"`
-		Devices                string   `schema:"devices"`
-		Dockerfile             string   `schema:"dockerfile"`
-		DropCapabilities       string   `schema:"dropcaps"`
-		Excludes               string   `schema:"excludes"`
-		ForceRm                bool     `schema:"forcerm"`
-		From                   string   `schema:"from"`
-		HTTPProxy              bool     `schema:"httpproxy"`
-		Ignore                 bool     `schema:"ignore"`
-		Isolation              string   `schema:"isolation"`
-		Jobs                   int      `schema:"jobs"` // nolint
-		LabelOpts              string   `schema:"labelopts"`
-		Labels                 string   `schema:"labels"`
-		Layers                 bool     `schema:"layers"`
-		LogRusage              bool     `schema:"rusage"`
-		Manifest               string   `schema:"manifest"`
-		MemSwap                int64    `schema:"memswap"`
-		Memory                 int64    `schema:"memory"`
-		NamespaceOptions       string   `schema:"nsoptions"`
-		NoCache                bool     `schema:"nocache"`
-		OutputFormat           string   `schema:"outputformat"`
-		Platform               []string `schema:"platform"`
-		Pull                   bool     `schema:"pull"`
-		PullPolicy             string   `schema:"pullpolicy"`
-		Quiet                  bool     `schema:"q"`
-		Registry               string   `schema:"registry"`
-		Rm                     bool     `schema:"rm"`
-		RusageLogFile          string   `schema:"rusagelogfile"`
-		Seccomp                string   `schema:"seccomp"`
-		SecurityOpt            string   `schema:"securityopt"`
-		ShmSize                int      `schema:"shmsize"`
-		Squash                 bool     `schema:"squash"`
-		Tag                    []string `schema:"t"`
-		Target                 string   `schema:"target"`
-		Timestamp              int64    `schema:"timestamp"`
-		Ulimits                string   `schema:"ulimits"`
+		AddHosts                string   `schema:"extrahosts"`
+		AdditionalCapabilities  string   `schema:"addcaps"`
+		AdditionalBuildContexts string   `schema:"additionalbuildcontexts"`
+		AllPlatforms            bool     `schema:"allplatforms"`
+		Annotations             string   `schema:"annotations"`
+		AppArmor                string   `schema:"apparmor"`
+		BuildArgs               string   `schema:"buildargs"`
+		CacheFrom               string   `schema:"cachefrom"`
+		CacheTo                 string   `schema:"cacheto"`
+		CacheTTL                string   `schema:"cachettl"`
+		CgroupParent            string   `schema:"cgroupparent"`
+		Compression             uint64   `schema:"compression"`
+		ConfigureNetwork        string   `schema:"networkmode"`
+		CPPFlags                string   `schema:"cppflags"`
+		CpuPeriod               uint64   `schema:"cpuperiod"`  //nolint:revive,stylecheck
+		CpuQuota                int64    `schema:"cpuquota"`   //nolint:revive,stylecheck
+		CpuSetCpus              string   `schema:"cpusetcpus"` //nolint:revive,stylecheck
+		CpuSetMems              string   `schema:"cpusetmems"` //nolint:revive,stylecheck
+		CpuShares               uint64   `schema:"cpushares"`  //nolint:revive,stylecheck
+		DNSOptions              string   `schema:"dnsoptions"`
+		DNSSearch               string   `schema:"dnssearch"`
+		DNSServers              string   `schema:"dnsservers"`
+		Devices                 string   `schema:"devices"`
+		Dockerfile              string   `schema:"dockerfile"`
+		DropCapabilities        string   `schema:"dropcaps"`
+		Envs                    []string `schema:"setenv"`
+		Excludes                string   `schema:"excludes"`
+		ForceRm                 bool     `schema:"forcerm"`
+		From                    string   `schema:"from"`
+		HTTPProxy               bool     `schema:"httpproxy"`
+		IDMappingOptions        string   `schema:"idmappingoptions"`
+		IdentityLabel           bool     `schema:"identitylabel"`
+		Ignore                  bool     `schema:"ignore"`
+		Isolation               string   `schema:"isolation"`
+		Jobs                    int      `schema:"jobs"`
+		LabelOpts               string   `schema:"labelopts"`
+		Labels                  string   `schema:"labels"`
+		Layers                  bool     `schema:"layers"`
+		LogRusage               bool     `schema:"rusage"`
+		Manifest                string   `schema:"manifest"`
+		MemSwap                 int64    `schema:"memswap"`
+		Memory                  int64    `schema:"memory"`
+		NamespaceOptions        string   `schema:"nsoptions"`
+		NoCache                 bool     `schema:"nocache"`
+		OmitHistory             bool     `schema:"omithistory"`
+		OSFeatures              []string `schema:"osfeature"`
+		OSVersion               string   `schema:"osversion"`
+		OutputFormat            string   `schema:"outputformat"`
+		Platform                []string `schema:"platform"`
+		Pull                    bool     `schema:"pull"`
+		PullPolicy              string   `schema:"pullpolicy"`
+		Quiet                   bool     `schema:"q"`
+		Registry                string   `schema:"registry"`
+		Rm                      bool     `schema:"rm"`
+		RusageLogFile           string   `schema:"rusagelogfile"`
+		Remote                  string   `schema:"remote"`
+		Seccomp                 string   `schema:"seccomp"`
+		Secrets                 string   `schema:"secrets"`
+		SecurityOpt             string   `schema:"securityopt"`
+		ShmSize                 int      `schema:"shmsize"`
+		SkipUnusedStages        bool     `schema:"skipunusedstages"`
+		Squash                  bool     `schema:"squash"`
+		TLSVerify               bool     `schema:"tlsVerify"`
+		Tags                    []string `schema:"t"`
+		Target                  string   `schema:"target"`
+		Timestamp               int64    `schema:"timestamp"`
+		Ulimits                 string   `schema:"ulimits"`
+		UnsetEnvs               []string `schema:"unsetenv"`
 	}{
-		Dockerfile: "Dockerfile",
-		Registry:   "docker.io",
-		Rm:         true,
-		ShmSize:    64 * 1024 * 1024,
+		Dockerfile:       "Dockerfile",
+		IdentityLabel:    true,
+		Registry:         "docker.io",
+		Rm:               true,
+		ShmSize:          64 * 1024 * 1024,
+		TLSVerify:        true,
+		SkipUnusedStages: true,
 	}
 
 	decoder := r.Context().Value(api.DecoderKey).(*schema.Decoder)
 	if err := decoder.Decode(&query, r.URL.Query()); err != nil {
-		utils.Error(w, http.StatusText(http.StatusBadRequest), http.StatusBadRequest, err)
+		utils.Error(w, http.StatusBadRequest, err)
 		return
 	}
 
 	// if layers field not set assume its not from a valid podman-client
 	// could be a docker client, set `layers=true` since that is the default
-	// expected behviour
+	// expected behaviour
 	if !utils.IsLibpodRequest(r) {
 		if _, found := r.URL.Query()["layers"]; !found {
 			query.Layers = true
 		}
 	}
 
+	// convert tag formats
+	tags := query.Tags
+
 	// convert addcaps formats
 	var addCaps = []string{}
 	if _, found := r.URL.Query()["addcaps"]; found {
@@ -157,14 +179,50 @@ func BuildImage(w http.ResponseWriter, r *http.Request) {
 
 	// convert addcaps formats
 	containerFiles := []string{}
-	if _, found := r.URL.Query()["dockerfile"]; found {
-		var m = []string{}
-		if err := json.Unmarshal([]byte(query.Dockerfile), &m); err != nil {
-			// it's not json, assume just a string
-			m = []string{filepath.Join(contextDirectory, query.Dockerfile)}
+	// Tells if query parameter `dockerfile` is set or not.
+	dockerFileSet := false
+	if utils.IsLibpodRequest(r) && query.Remote != "" {
+		// The context directory could be a URL.  Try to handle that.
+		anchorDir, err := os.MkdirTemp(parse.GetTempDir(), "libpod_builder")
+		if err != nil {
+			utils.InternalServerError(w, err)
+		}
+		tempDir, subDir, err := buildahDefine.TempDirForURL(anchorDir, "buildah", query.Remote)
+		if err != nil {
+			utils.InternalServerError(w, err)
+		}
+		if tempDir != "" {
+			// We had to download it to a temporary directory.
+			// Delete it later.
+			defer func() {
+				if err = os.RemoveAll(tempDir); err != nil {
+					// We are deleting this on server so log on server end
+					// client does not have to worry about server cleanup.
+					logrus.Errorf("Cannot delete downloaded temp dir %q: %s", tempDir, err)
+				}
+			}()
+			contextDirectory = filepath.Join(tempDir, subDir)
+		} else {
+			// Nope, it was local.  Use it as is.
+			absDir, err := filepath.Abs(query.Remote)
+			if err != nil {
+				utils.BadRequest(w, "remote", query.Remote, err)
+			}
+			contextDirectory = absDir
 		}
-		containerFiles = m
 	} else {
+		if _, found := r.URL.Query()["dockerfile"]; found {
+			var m = []string{}
+			if err := json.Unmarshal([]byte(query.Dockerfile), &m); err != nil {
+				// it's not json, assume just a string
+				m = []string{filepath.Join(contextDirectory, query.Dockerfile)}
+			}
+			containerFiles = m
+			dockerFileSet = true
+		}
+	}
+
+	if !dockerFileSet {
 		containerFiles = []string{filepath.Join(contextDirectory, "Dockerfile")}
 		if utils.IsLibpodRequest(r) {
 			containerFiles = []string{filepath.Join(contextDirectory, "Containerfile")}
@@ -239,15 +297,73 @@ func BuildImage(w http.ResponseWriter, r *http.Request) {
 		dnssearch = m
 	}
 
+	var secrets = []string{}
+	if _, found := r.URL.Query()["secrets"]; found {
+		var m = []string{}
+		if err := json.Unmarshal([]byte(query.Secrets), &m); err != nil {
+			utils.BadRequest(w, "secrets", query.Secrets, err)
+			return
+		}
+
+		// for podman-remote all secrets must be picked from context director
+		// hence modify src so contextdir is added as prefix
+
+		for _, secret := range m {
+			secretOpt := strings.Split(secret, ",")
+			if len(secretOpt) > 0 {
+				modifiedOpt := []string{}
+				for _, token := range secretOpt {
+					arr := strings.SplitN(token, "=", 2)
+					if len(arr) > 1 {
+						if arr[0] == "src" {
+							/* move secret away from contextDir */
+							/* to make sure we dont accidentally commit temporary secrets to image*/
+							builderDirectory, _ := filepath.Split(contextDirectory)
+							// following path is outside build context
+							newSecretPath := filepath.Join(builderDirectory, arr[1])
+							oldSecretPath := filepath.Join(contextDirectory, arr[1])
+							err := os.Rename(oldSecretPath, newSecretPath)
+							if err != nil {
+								utils.BadRequest(w, "secrets", query.Secrets, err)
+								return
+							}
+
+							modifiedSrc := fmt.Sprintf("src=%s", newSecretPath)
+							modifiedOpt = append(modifiedOpt, modifiedSrc)
+						} else {
+							modifiedOpt = append(modifiedOpt, token)
+						}
+					}
+				}
+				secrets = append(secrets, strings.Join(modifiedOpt, ","))
+			}
+		}
+	}
+
 	var output string
-	if len(query.Tag) > 0 {
-		output = query.Tag[0]
+	if len(tags) > 0 {
+		possiblyNormalizedName, err := utils.NormalizeToDockerHub(r, tags[0])
+		if err != nil {
+			utils.Error(w, http.StatusInternalServerError, fmt.Errorf("normalizing image: %w", err))
+			return
+		}
+		output = possiblyNormalizedName
 	}
 	format := buildah.Dockerv2ImageManifest
 	registry := query.Registry
 	isolation := buildah.IsolationDefault
 	if utils.IsLibpodRequest(r) {
-		isolation = parseLibPodIsolation(query.Isolation)
+		var err error
+		isolation, err = parseLibPodIsolation(query.Isolation)
+		if err != nil {
+			utils.Error(w, http.StatusInternalServerError, fmt.Errorf("failed to parse isolation: %w", err))
+			return
+		}
+
+		// make sure to force rootless as rootless otherwise buildah runs code which is intended to be run only as root.
+		if isolation == buildah.IsolationOCI && rootless.IsRootless() {
+			isolation = buildah.IsolationOCIRootless
+		}
 		registry = ""
 		format = query.OutputFormat
 	} else {
@@ -258,8 +374,54 @@ func BuildImage(w http.ResponseWriter, r *http.Request) {
 		}
 	}
 	var additionalTags []string
-	if len(query.Tag) > 1 {
-		additionalTags = query.Tag[1:]
+	for i := 1; i < len(tags); i++ {
+		possiblyNormalizedTag, err := utils.NormalizeToDockerHub(r, tags[i])
+		if err != nil {
+			utils.Error(w, http.StatusInternalServerError, fmt.Errorf("normalizing image: %w", err))
+			return
+		}
+		additionalTags = append(additionalTags, possiblyNormalizedTag)
+	}
+
+	var additionalBuildContexts = map[string]*buildahDefine.AdditionalBuildContext{}
+	if _, found := r.URL.Query()["additionalbuildcontexts"]; found {
+		if err := json.Unmarshal([]byte(query.AdditionalBuildContexts), &additionalBuildContexts); err != nil {
+			utils.BadRequest(w, "additionalbuildcontexts", query.AdditionalBuildContexts, err)
+			return
+		}
+	}
+
+	var idMappingOptions buildahDefine.IDMappingOptions
+	if _, found := r.URL.Query()["idmappingoptions"]; found {
+		if err := json.Unmarshal([]byte(query.IDMappingOptions), &idMappingOptions); err != nil {
+			utils.BadRequest(w, "idmappingoptions", query.IDMappingOptions, err)
+			return
+		}
+	}
+
+	var cacheFrom reference.Named
+	if _, found := r.URL.Query()["cachefrom"]; found {
+		cacheFrom, err = parse.RepoNameToNamedReference(query.CacheFrom)
+		if err != nil {
+			utils.BadRequest(w, "cacheFrom", query.CacheFrom, err)
+			return
+		}
+	}
+	var cacheTo reference.Named
+	if _, found := r.URL.Query()["cacheto"]; found {
+		cacheTo, err = parse.RepoNameToNamedReference(query.CacheTo)
+		if err != nil {
+			utils.BadRequest(w, "cacheto", query.CacheTo, err)
+			return
+		}
+	}
+	var cacheTTL time.Duration
+	if _, found := r.URL.Query()["cachettl"]; found {
+		cacheTTL, err = time.ParseDuration(query.CacheTTL)
+		if err != nil {
+			utils.BadRequest(w, "cachettl", query.CacheTTL, err)
+			return
+		}
 	}
 
 	var buildArgs = map[string]string{}
@@ -287,6 +449,15 @@ func BuildImage(w http.ResponseWriter, r *http.Request) {
 		}
 	}
 
+	// convert cppflags formats
+	var cppflags = []string{}
+	if _, found := r.URL.Query()["cppflags"]; found {
+		if err := json.Unmarshal([]byte(query.CPPFlags), &cppflags); err != nil {
+			utils.BadRequest(w, "cppflags", query.CPPFlags, err)
+			return
+		}
+	}
+
 	// convert nsoptions formats
 	nsoptions := buildah.NamespaceOptions{}
 	if _, found := r.URL.Query()["nsoptions"]; found {
@@ -355,7 +526,7 @@ func BuildImage(w http.ResponseWriter, r *http.Request) {
 				}
 				con := strings.SplitN(opt, "=", 2)
 				if len(con) != 2 {
-					utils.BadRequest(w, "securityopt", query.SecurityOpt, errors.Errorf("Invalid --security-opt name=value pair: %q", opt))
+					utils.BadRequest(w, "securityopt", query.SecurityOpt, fmt.Errorf("invalid --security-opt name=value pair: %q", opt))
 					return
 				}
 
@@ -367,7 +538,7 @@ func BuildImage(w http.ResponseWriter, r *http.Request) {
 				case "seccomp":
 					seccomp = con[1]
 				default:
-					utils.BadRequest(w, "securityopt", query.SecurityOpt, errors.Errorf("Invalid --security-opt 2: %q", opt))
+					utils.BadRequest(w, "securityopt", query.SecurityOpt, fmt.Errorf("invalid --security-opt 2: %q", opt))
 					return
 				}
 			}
@@ -396,14 +567,35 @@ func BuildImage(w http.ResponseWriter, r *http.Request) {
 		}
 	}
 
-	creds, authfile, key, err := auth.GetCredentials(r)
+	creds, authfile, err := auth.GetCredentials(r)
 	if err != nil {
 		// Credential value(s) not returned as their value is not human readable
-		utils.BadRequest(w, key.String(), "n/a", err)
+		utils.Error(w, http.StatusBadRequest, err)
 		return
 	}
 	defer auth.RemoveAuthfile(authfile)
 
+	fromImage := query.From
+	if fromImage != "" {
+		possiblyNormalizedName, err := utils.NormalizeToDockerHub(r, fromImage)
+		if err != nil {
+			utils.Error(w, http.StatusInternalServerError, fmt.Errorf("normalizing image: %w", err))
+			return
+		}
+		fromImage = possiblyNormalizedName
+	}
+
+	systemContext := &types.SystemContext{
+		AuthFilePath:     authfile,
+		DockerAuthConfig: creds,
+	}
+	utils.PossiblyEnforceDockerHub(r, systemContext)
+
+	if _, found := r.URL.Query()["tlsVerify"]; found {
+		systemContext.DockerInsecureSkipTLSVerify = types.NewOptionalBool(!query.TLSVerify)
+		systemContext.OCIInsecureSkipTLSVerify = !query.TLSVerify
+		systemContext.DockerDaemonInsecureSkipTLSVerify = !query.TLSVerify
+	}
 	// Channels all mux'ed in select{} below to follow API build protocol
 	stdout := channel.NewWriter(make(chan []byte))
 	defer stdout.Close()
@@ -418,16 +610,17 @@ func BuildImage(w http.ResponseWriter, r *http.Request) {
 	defer reporter.Close()
 
 	runtime := r.Context().Value(api.RuntimeKey).(*libpod.Runtime)
-	rtc, err := runtime.GetConfig()
-	if err != nil {
-		utils.Error(w, "Something went wrong.", http.StatusInternalServerError, errors.Wrap(err, "Decode()"))
-		return
-	}
 	buildOptions := buildahDefine.BuildOptions{
-		AddCapabilities: addCaps,
-		AdditionalTags:  additionalTags,
-		Annotations:     annotations,
-		Args:            buildArgs,
+		AddCapabilities:         addCaps,
+		AdditionalBuildContexts: additionalBuildContexts,
+		AdditionalTags:          additionalTags,
+		Annotations:             annotations,
+		CPPFlags:                cppflags,
+		CacheFrom:               cacheFrom,
+		CacheTo:                 cacheTo,
+		CacheTTL:                cacheTTL,
+		Args:                    buildArgs,
+		AllPlatforms:            query.AllPlatforms,
 		CommonBuildOpts: &buildah.CommonBuildOptions{
 			AddHost:            addhosts,
 			ApparmorProfile:    apparmor,
@@ -441,24 +634,27 @@ func BuildImage(w http.ResponseWriter, r *http.Request) {
 			DNSSearch:          dnssearch,
 			DNSServers:         dnsservers,
 			HTTPProxy:          query.HTTPProxy,
+			IdentityLabel:      types.NewOptionalBool(query.IdentityLabel),
 			LabelOpts:          labelOpts,
 			Memory:             query.Memory,
 			MemorySwap:         query.MemSwap,
+			OmitHistory:        query.OmitHistory,
 			SeccompProfilePath: seccomp,
 			ShmSize:            strconv.Itoa(query.ShmSize),
 			Ulimit:             ulimits,
+			Secrets:            secrets,
 		},
-		CNIConfigDir:                   rtc.Network.CNIPluginDirs[0],
-		CNIPluginPath:                  util.DefaultCNIPluginPath,
 		Compression:                    compression,
 		ConfigureNetwork:               parseNetworkConfigurationPolicy(query.ConfigureNetwork),
 		ContextDirectory:               contextDirectory,
 		Devices:                        devices,
 		DropCapabilities:               dropCaps,
+		Envs:                           query.Envs,
 		Err:                            auxout,
 		Excludes:                       excludes,
 		ForceRmIntermediateCtrs:        query.ForceRm,
-		From:                           query.From,
+		From:                           fromImage,
+		IDMappingOptions:               &idMappingOptions,
 		IgnoreUnrecognizedInstructions: query.Ignore,
 		Isolation:                      isolation,
 		Jobs:                           &jobs,
@@ -469,22 +665,23 @@ func BuildImage(w http.ResponseWriter, r *http.Request) {
 		MaxPullPushRetries:             3,
 		NamespaceOptions:               nsoptions,
 		NoCache:                        query.NoCache,
+		OSFeatures:                     query.OSFeatures,
+		OSVersion:                      query.OSVersion,
 		Out:                            stdout,
 		Output:                         output,
 		OutputFormat:                   format,
 		PullPolicy:                     pullPolicy,
-		PullPushRetryDelay:             time.Duration(2 * time.Second),
+		PullPushRetryDelay:             2 * time.Second,
 		Quiet:                          query.Quiet,
 		Registry:                       registry,
 		RemoveIntermediateCtrs:         query.Rm,
 		ReportWriter:                   reporter,
 		RusageLogFile:                  query.RusageLogFile,
+		SkipUnusedStages:               types.NewOptionalBool(query.SkipUnusedStages),
 		Squash:                         query.Squash,
+		SystemContext:                  systemContext,
 		Target:                         query.Target,
-		SystemContext: &types.SystemContext{
-			AuthFilePath:     authfile,
-			DockerAuthConfig: creds,
-		},
+		UnsetEnvs:                      query.UnsetEnvs,
 	}
 
 	for _, platformSpec := range query.Platform {
@@ -509,7 +706,7 @@ func BuildImage(w http.ResponseWriter, r *http.Request) {
 		success bool
 	)
 
-	runCtx, cancel := context.WithCancel(context.Background())
+	runCtx, cancel := context.WithCancel(r.Context())
 	go func() {
 		defer cancel()
 		imageID, _, err = runtime.Build(r.Context(), buildOptions, containerFiles...)
@@ -527,15 +724,15 @@ func BuildImage(w http.ResponseWriter, r *http.Request) {
 	}
 
 	// Send headers and prime client for stream to come
-	w.WriteHeader(http.StatusOK)
 	w.Header().Set("Content-Type", "application/json")
+	w.WriteHeader(http.StatusOK)
 	flush()
 
 	body := w.(io.Writer)
 	if logrus.IsLevelEnabled(logrus.DebugLevel) {
 		if v, found := os.LookupEnv("PODMAN_RETAIN_BUILD_ARTIFACT"); found {
 			if keep, _ := strconv.ParseBool(v); keep {
-				t, _ := ioutil.TempFile("", "build_*_server")
+				t, _ := os.CreateTemp("", "build_*_server")
 				defer t.Close()
 				body = io.MultiWriter(t, w)
 			}
@@ -544,14 +741,17 @@ func BuildImage(w http.ResponseWriter, r *http.Request) {
 
 	enc := json.NewEncoder(body)
 	enc.SetEscapeHTML(true)
+	var stepErrors []string
 
 	for {
-		m := struct {
+		type BuildResponse struct {
 			Stream string                 `json:"stream,omitempty"`
 			Error  *jsonmessage.JSONError `json:"errorDetail,omitempty"`
 			// NOTE: `error` is being deprecated check https://github.com/moby/moby/blob/master/pkg/jsonmessage/jsonmessage.go#L148
-			ErrorMessage string `json:"error,omitempty"` // deprecate this slowly
-		}{}
+			ErrorMessage string          `json:"error,omitempty"` // deprecate this slowly
+			Aux          json.RawMessage `json:"aux,omitempty"`
+		}
+		m := BuildResponse{}
 
 		select {
 		case e := <-stdout.Chan():
@@ -567,12 +767,27 @@ func BuildImage(w http.ResponseWriter, r *http.Request) {
 			}
 			flush()
 		case e := <-auxout.Chan():
-			m.Stream = string(e)
-			if err := enc.Encode(m); err != nil {
-				stderr.Write([]byte(err.Error()))
+			if !query.Quiet {
+				m.Stream = string(e)
+				if err := enc.Encode(m); err != nil {
+					stderr.Write([]byte(err.Error()))
+				}
+				flush()
+			} else {
+				stepErrors = append(stepErrors, string(e))
 			}
-			flush()
 		case e := <-stderr.Chan():
+			// Docker-API Compat parity : Build failed so
+			// output all step errors irrespective of quiet
+			// flag.
+			for _, stepError := range stepErrors {
+				t := BuildResponse{}
+				t.Stream = stepError
+				if err := enc.Encode(t); err != nil {
+					stderr.Write([]byte(err.Error()))
+				}
+				flush()
+			}
 			m.ErrorMessage = string(e)
 			m.Error = &jsonmessage.JSONError{
 				Message: m.ErrorMessage,
@@ -584,13 +799,18 @@ func BuildImage(w http.ResponseWriter, r *http.Request) {
 			return
 		case <-runCtx.Done():
 			if success {
-				if !utils.IsLibpodRequest(r) {
+				if !utils.IsLibpodRequest(r) && !query.Quiet {
+					m.Aux = []byte(fmt.Sprintf(`{"ID":"sha256:%s"}`, imageID))
+					if err := enc.Encode(m); err != nil {
+						logrus.Warnf("failed to json encode error %v", err)
+					}
+					m.Aux = nil
 					m.Stream = fmt.Sprintf("Successfully built %12.12s\n", imageID)
 					if err := enc.Encode(m); err != nil {
 						logrus.Warnf("Failed to json encode error %v", err)
 					}
 					flush()
-					for _, tag := range query.Tag {
+					for _, tag := range tags {
 						m.Stream = fmt.Sprintf("Successfully tagged %s\n", tag)
 						if err := enc.Encode(m); err != nil {
 							logrus.Warnf("Failed to json encode error %v", err)
@@ -625,33 +845,22 @@ func parseNetworkConfigurationPolicy(network string) buildah.NetworkConfiguratio
 	}
 }
 
-func parseLibPodIsolation(isolation string) buildah.Isolation { // nolint
+func parseLibPodIsolation(isolation string) (buildah.Isolation, error) {
 	if val, err := strconv.Atoi(isolation); err == nil {
-		return buildah.Isolation(val)
-	}
-	switch isolation {
-	case "IsolationDefault", "default":
-		return buildah.IsolationDefault
-	case "IsolationOCI":
-		return buildah.IsolationOCI
-	case "IsolationChroot":
-		return buildah.IsolationChroot
-	case "IsolationOCIRootless":
-		return buildah.IsolationOCIRootless
-	default:
-		return buildah.IsolationDefault
+		return buildah.Isolation(val), nil
 	}
+	return parse.IsolationOption(isolation)
 }
 
 func extractTarFile(r *http.Request) (string, error) {
 	// build a home for the request body
-	anchorDir, err := ioutil.TempDir("", "libpod_builder")
+	anchorDir, err := os.MkdirTemp("", "libpod_builder")
 	if err != nil {
 		return "", err
 	}
 
 	path := filepath.Join(anchorDir, "tarBall")
-	tarBall, err := os.OpenFile(path, os.O_RDWR|os.O_CREATE|os.O_TRUNC, 0600)
+	tarBall, err := os.OpenFile(path, os.O_RDWR|os.O_CREATE|os.O_TRUNC, 0o600)
 	if err != nil {
 		return "", err
 	}
@@ -665,7 +874,7 @@ func extractTarFile(r *http.Request) (string, error) {
 	}
 
 	buildDir := filepath.Join(anchorDir, "build")
-	err = os.Mkdir(buildDir, 0700)
+	err = os.Mkdir(buildDir, 0o700)
 	if err != nil {
 		return "", err
 	}
