@@ -1,11 +1,11 @@
 package utils
 
 import (
+	"fmt"
 	"strings"
 
 	"github.com/containers/common/pkg/secrets"
-	"github.com/containers/podman/v3/pkg/util"
-	"github.com/pkg/errors"
+	"github.com/containers/podman/v4/pkg/util"
 )
 
 func IfPassesSecretsFilter(s secrets.Secret, filters map[string][]string) (bool, error) {
@@ -17,7 +17,7 @@ func IfPassesSecretsFilter(s secrets.Secret, filters map[string][]string) (bool,
 		case "id":
 			result = util.StringMatchRegexSlice(s.ID, filterValues)
 		default:
-			return false, errors.Errorf("invalid filter %q", key)
+			return false, fmt.Errorf("invalid filter %q", key)
 		}
 	}
 	return result, nil
