@@ -1,6 +1,8 @@
 package ps
 
 import (
+	"errors"
+	"fmt"
 	"os"
 	"path/filepath"
 	"regexp"
@@ -9,13 +11,13 @@ import (
 	"strings"
 	"time"
 
-	"github.com/containers/podman/v3/libpod"
-	"github.com/containers/podman/v3/libpod/define"
-	"github.com/containers/podman/v3/pkg/domain/entities"
-	"github.com/containers/podman/v3/pkg/domain/filters"
-	psdefine "github.com/containers/podman/v3/pkg/ps/define"
+	"github.com/containers/podman/v4/libpod"
+	"github.com/containers/podman/v4/libpod/define"
+	"github.com/containers/podman/v4/pkg/domain/entities"
+	"github.com/containers/podman/v4/pkg/domain/filters"
+	psdefine "github.com/containers/podman/v4/pkg/ps/define"
 	"github.com/containers/storage"
-	"github.com/pkg/errors"
+	"github.com/containers/storage/types"
 	"github.com/sirupsen/logrus"
 )
 
@@ -63,13 +65,17 @@ func GetContainerLists(runtime *libpod.Runtime, options entities.ContainerListOp
 	}
 	for _, con := range cons {
 		listCon, err := ListContainerBatch(runtime, con, options)
-		if err != nil {
+		switch {
+		case errors.Is(err, define.ErrNoSuchCtr):
+			continue
+		case err != nil:
 			return nil, err
+		default:
+			pss = append(pss, listCon)
 		}
-		pss = append(pss, listCon)
 	}
 
-	if options.All && options.External {
+	if options.External {
 		listCon, err := GetExternalContainerLists(runtime)
 		if err != nil {
 			return nil, err
@@ -89,7 +95,7 @@ func GetContainerLists(runtime *libpod.Runtime, options entities.ContainerListOp
 	return pss, nil
 }
 
-// GetExternalContainerLists returns list of external containers for e.g created by buildah
+// GetExternalContainerLists returns list of external containers for e.g. created by buildah
 func GetExternalContainerLists(runtime *libpod.Runtime) ([]entities.ListContainer, error) {
 	var (
 		pss = []entities.ListContainer{}
@@ -102,15 +108,19 @@ func GetExternalContainerLists(runtime *libpod.Runtime) ([]entities.ListContaine
 
 	for _, con := range externCons {
 		listCon, err := ListStorageContainer(runtime, con)
-		if err != nil {
+		switch {
+		case errors.Is(err, types.ErrLoadError):
+			continue
+		case err != nil:
 			return nil, err
+		default:
+			pss = append(pss, listCon)
 		}
-		pss = append(pss, listCon)
 	}
 	return pss, nil
 }
 
-// BatchContainerOp is used in ps to reduce performance hits by "batching"
+// ListContainerBatch is used in ps to reduce performance hits by "batching"
 // locks.
 func ListContainerBatch(rt *libpod.Runtime, ctr *libpod.Container, opts entities.ContainerListOptions) (entities.ListContainer, error) {
 	var (
@@ -129,32 +139,32 @@ func ListContainerBatch(rt *libpod.Runtime, ctr *libpod.Container, opts entities
 	batchErr := ctr.Batch(func(c *libpod.Container) error {
 		if opts.Sync {
 			if err := c.Sync(); err != nil {
-				return errors.Wrapf(err, "unable to update container state from OCI runtime")
+				return fmt.Errorf("unable to update container state from OCI runtime: %w", err)
 			}
 		}
 
 		conConfig = c.Config()
 		conState, err = c.State()
 		if err != nil {
-			return errors.Wrapf(err, "unable to obtain container state")
+			return fmt.Errorf("unable to obtain container state: %w", err)
 		}
 
 		exitCode, exited, err = c.ExitCode()
 		if err != nil {
-			return errors.Wrapf(err, "unable to obtain container exit code")
+			return fmt.Errorf("unable to obtain container exit code: %w", err)
 		}
 		startedTime, err = c.StartedTime()
 		if err != nil {
-			logrus.Errorf("error getting started time for %q: %v", c.ID(), err)
+			logrus.Errorf("Getting started time for %q: %v", c.ID(), err)
 		}
 		exitedTime, err = c.FinishedTime()
 		if err != nil {
-			logrus.Errorf("error getting exited time for %q: %v", c.ID(), err)
+			logrus.Errorf("Getting exited time for %q: %v", c.ID(), err)
 		}
 
 		pid, err = c.PID()
 		if err != nil {
-			return errors.Wrapf(err, "unable to obtain container pid")
+			return fmt.Errorf("unable to obtain container pid: %w", err)
 		}
 
 		if !opts.Size && !opts.Namespace {
@@ -176,12 +186,12 @@ func ListContainerBatch(rt *libpod.Runtime, ctr *libpod.Container, opts entities
 
 			rootFsSize, err := c.RootFsSize()
 			if err != nil {
-				logrus.Errorf("error getting root fs size for %q: %v", c.ID(), err)
+				logrus.Errorf("Getting root fs size for %q: %v", c.ID(), err)
 			}
 
 			rwSize, err := c.RWSize()
 			if err != nil {
-				logrus.Errorf("error getting rw size for %q: %v", c.ID(), err)
+				logrus.Errorf("Getting rw size for %q: %v", c.ID(), err)
 			}
 
 			size.RootFsSize = rootFsSize
@@ -198,7 +208,7 @@ func ListContainerBatch(rt *libpod.Runtime, ctr *libpod.Container, opts entities
 		return entities.ListContainer{}, err
 	}
 
-	networks, _, err := ctr.Networks()
+	networks, err := ctr.Networks()
 	if err != nil {
 		return entities.ListContainer{}, err
 	}
@@ -228,8 +238,8 @@ func ListContainerBatch(rt *libpod.Runtime, ctr *libpod.Container, opts entities
 	if opts.Pod && len(conConfig.Pod) > 0 {
 		podName, err := rt.GetName(conConfig.Pod)
 		if err != nil {
-			if errors.Cause(err) == define.ErrNoSuchCtr {
-				return entities.ListContainer{}, errors.Wrapf(define.ErrNoSuchPod, "could not find container %s pod (id %s) in state", conConfig.ID, conConfig.Pod)
+			if errors.Is(err, define.ErrNoSuchCtr) {
+				return entities.ListContainer{}, fmt.Errorf("could not find container %s pod (id %s) in state: %w", conConfig.ID, conConfig.Pod, define.ErrNoSuchPod)
 			}
 			return entities.ListContainer{}, err
 		}
@@ -273,7 +283,7 @@ func ListStorageContainer(rt *libpod.Runtime, ctr storage.Container) (entities.L
 
 	buildahCtr, err := rt.IsBuildahContainer(ctr.ID)
 	if err != nil {
-		return ps, errors.Wrapf(err, "error determining buildah container for container %s", ctr.ID)
+		return ps, fmt.Errorf("determining buildah container for container %s: %w", ctr.ID, err)
 	}
 
 	if buildahCtr {
@@ -302,7 +312,7 @@ func ListStorageContainer(rt *libpod.Runtime, ctr storage.Container) (entities.L
 func getNamespaceInfo(path string) (string, error) {
 	val, err := os.Readlink(path)
 	if err != nil {
-		return "", errors.Wrapf(err, "error getting info from %q", path)
+		return "", fmt.Errorf("getting info from %q: %w", path, err)
 	}
 	return getStrFromSquareBrackets(val), nil
 }
