@@ -1,23 +1,23 @@
 package specgenutil
 
 import (
+	"encoding/csv"
+	"errors"
 	"fmt"
-	"path/filepath"
+	"path"
 	"strings"
 
 	"github.com/containers/common/pkg/parse"
-	"github.com/containers/podman/v3/libpod/define"
-	"github.com/containers/podman/v3/pkg/specgen"
-	"github.com/containers/podman/v3/pkg/util"
+	"github.com/containers/podman/v4/libpod/define"
+	"github.com/containers/podman/v4/pkg/specgen"
+	"github.com/containers/podman/v4/pkg/util"
 	spec "github.com/opencontainers/runtime-spec/specs-go"
-	"github.com/pkg/errors"
 )
 
 var (
-	errDuplicateDest = errors.Errorf("duplicate mount destination")
-	optionArgError   = errors.Errorf("must provide an argument for option")
-	noDestError      = errors.Errorf("must set volume destination")
-	errInvalidSyntax = errors.Errorf("incorrect mount format: should be --mount type=<bind|tmpfs|volume>,[src=<host-dir|volume-name>,]target=<ctr-dir>[,options]")
+	errOptionArg     = errors.New("must provide an argument for option")
+	errNoDest        = errors.New("must set volume destination")
+	errInvalidSyntax = errors.New("incorrect mount format: should be --mount type=<bind|tmpfs|volume>,[src=<host-dir|volume-name>,]target=<ctr-dir>[,options]")
 )
 
 // Parse all volume-related options in the create config into a set of mounts
@@ -28,7 +28,7 @@ var (
 // TODO: handle options parsing/processing via containers/storage/pkg/mount
 func parseVolumes(volumeFlag, mountFlag, tmpfsFlag []string, addReadOnlyTmpfs bool) ([]spec.Mount, []*specgen.NamedVolume, []*specgen.OverlayVolume, []*specgen.ImageVolume, error) {
 	// Get mounts from the --mounts flag.
-	unifiedMounts, unifiedVolumes, unifiedImageVolumes, err := getMounts(mountFlag)
+	unifiedMounts, unifiedVolumes, unifiedImageVolumes, err := Mounts(mountFlag)
 	if err != nil {
 		return nil, nil, nil, nil, err
 	}
@@ -48,21 +48,32 @@ func parseVolumes(volumeFlag, mountFlag, tmpfsFlag []string, addReadOnlyTmpfs bo
 	// Unify mounts from --mount, --volume, --tmpfs.
 	// Start with --volume.
 	for dest, mount := range volumeMounts {
-		if _, ok := unifiedMounts[dest]; ok {
-			return nil, nil, nil, nil, errors.Wrapf(errDuplicateDest, dest)
+		if vol, ok := unifiedMounts[dest]; ok {
+			if mount.Source == vol.Source &&
+				specgen.StringSlicesEqual(vol.Options, mount.Options) {
+				continue
+			}
+			return nil, nil, nil, nil, fmt.Errorf("%v: %w", dest, specgen.ErrDuplicateDest)
 		}
 		unifiedMounts[dest] = mount
 	}
 	for dest, volume := range volumeVolumes {
-		if _, ok := unifiedVolumes[dest]; ok {
-			return nil, nil, nil, nil, errors.Wrapf(errDuplicateDest, dest)
+		if vol, ok := unifiedVolumes[dest]; ok {
+			if volume.Name == vol.Name &&
+				specgen.StringSlicesEqual(vol.Options, volume.Options) {
+				continue
+			}
+			return nil, nil, nil, nil, fmt.Errorf("%v: %w", dest, specgen.ErrDuplicateDest)
 		}
 		unifiedVolumes[dest] = volume
 	}
 	// Now --tmpfs
 	for dest, tmpfs := range tmpfsMounts {
-		if _, ok := unifiedMounts[dest]; ok {
-			return nil, nil, nil, nil, errors.Wrapf(errDuplicateDest, dest)
+		if vol, ok := unifiedMounts[dest]; ok {
+			if vol.Type != define.TypeTmpfs {
+				return nil, nil, nil, nil, fmt.Errorf("%v: %w", dest, specgen.ErrDuplicateDest)
+			}
+			continue
 		}
 		unifiedMounts[dest] = tmpfs
 	}
@@ -92,7 +103,7 @@ func parseVolumes(volumeFlag, mountFlag, tmpfsFlag []string, addReadOnlyTmpfs bo
 	allMounts := make(map[string]bool)
 	testAndSet := func(dest string) error {
 		if _, ok := allMounts[dest]; ok {
-			return errors.Wrapf(errDuplicateDest, "conflict at mount destination %v", dest)
+			return fmt.Errorf("%v: %w", dest, specgen.ErrDuplicateDest)
 		}
 		allMounts[dest] = true
 		return nil
@@ -122,9 +133,9 @@ func parseVolumes(volumeFlag, mountFlag, tmpfsFlag []string, addReadOnlyTmpfs bo
 	finalMounts := make([]spec.Mount, 0, len(unifiedMounts))
 	for _, mount := range unifiedMounts {
 		if mount.Type == define.TypeBind {
-			absSrc, err := filepath.Abs(mount.Source)
+			absSrc, err := specgen.ConvertWinMountPath(mount.Source)
 			if err != nil {
-				return nil, nil, nil, nil, errors.Wrapf(err, "error getting absolute path of %s", mount.Source)
+				return nil, nil, nil, nil, fmt.Errorf("getting absolute path of %s: %w", mount.Source, err)
 			}
 			mount.Source = absSrc
 		}
@@ -152,7 +163,15 @@ func findMountType(input string) (mountType string, tokens []string, err error)
 	// Split by comma, iterate over the slice and look for
 	// "type=$mountType". Everything else is appended to tokens.
 	found := false
-	for _, s := range strings.Split(input, ",") {
+	csvReader := csv.NewReader(strings.NewReader(input))
+	records, err := csvReader.ReadAll()
+	if err != nil {
+		return "", nil, err
+	}
+	if len(records) != 1 {
+		return "", nil, errInvalidSyntax
+	}
+	for _, s := range records[0] {
 		kv := strings.Split(s, "=")
 		if found || !(len(kv) == 2 && kv[0] == "type") {
 			tokens = append(tokens, s)
@@ -167,12 +186,12 @@ func findMountType(input string) (mountType string, tokens []string, err error)
 	return
 }
 
-// getMounts takes user-provided input from the --mount flag and creates OCI
+// Mounts takes user-provided input from the --mount flag and creates OCI
 // spec mounts and Libpod named volumes.
 // podman run --mount type=bind,src=/etc/resolv.conf,target=/etc/resolv.conf ...
 // podman run --mount type=tmpfs,target=/dev/shm ...
 // podman run --mount type=volume,source=test-volume, ...
-func getMounts(mountFlag []string) (map[string]spec.Mount, map[string]*specgen.NamedVolume, map[string]*specgen.ImageVolume, error) {
+func Mounts(mountFlag []string) (map[string]spec.Mount, map[string]*specgen.NamedVolume, map[string]*specgen.ImageVolume, error) {
 	finalMounts := make(map[string]spec.Mount)
 	finalNamedVolumes := make(map[string]*specgen.NamedVolume)
 	finalImageVolumes := make(map[string]*specgen.ImageVolume)
@@ -190,7 +209,7 @@ func getMounts(mountFlag []string) (map[string]spec.Mount, map[string]*specgen.N
 				return nil, nil, nil, err
 			}
 			if _, ok := finalMounts[mount.Destination]; ok {
-				return nil, nil, nil, errors.Wrapf(errDuplicateDest, mount.Destination)
+				return nil, nil, nil, fmt.Errorf("%v: %w", mount.Destination, specgen.ErrDuplicateDest)
 			}
 			finalMounts[mount.Destination] = mount
 		case define.TypeTmpfs:
@@ -199,7 +218,7 @@ func getMounts(mountFlag []string) (map[string]spec.Mount, map[string]*specgen.N
 				return nil, nil, nil, err
 			}
 			if _, ok := finalMounts[mount.Destination]; ok {
-				return nil, nil, nil, errors.Wrapf(errDuplicateDest, mount.Destination)
+				return nil, nil, nil, fmt.Errorf("%v: %w", mount.Destination, specgen.ErrDuplicateDest)
 			}
 			finalMounts[mount.Destination] = mount
 		case define.TypeDevpts:
@@ -208,7 +227,7 @@ func getMounts(mountFlag []string) (map[string]spec.Mount, map[string]*specgen.N
 				return nil, nil, nil, err
 			}
 			if _, ok := finalMounts[mount.Destination]; ok {
-				return nil, nil, nil, errors.Wrapf(errDuplicateDest, mount.Destination)
+				return nil, nil, nil, fmt.Errorf("%v: %w", mount.Destination, specgen.ErrDuplicateDest)
 			}
 			finalMounts[mount.Destination] = mount
 		case "image":
@@ -217,7 +236,7 @@ func getMounts(mountFlag []string) (map[string]spec.Mount, map[string]*specgen.N
 				return nil, nil, nil, err
 			}
 			if _, ok := finalImageVolumes[volume.Destination]; ok {
-				return nil, nil, nil, errors.Wrapf(errDuplicateDest, volume.Destination)
+				return nil, nil, nil, fmt.Errorf("%v: %w", volume.Destination, specgen.ErrDuplicateDest)
 			}
 			finalImageVolumes[volume.Destination] = volume
 		case "volume":
@@ -226,11 +245,11 @@ func getMounts(mountFlag []string) (map[string]spec.Mount, map[string]*specgen.N
 				return nil, nil, nil, err
 			}
 			if _, ok := finalNamedVolumes[volume.Dest]; ok {
-				return nil, nil, nil, errors.Wrapf(errDuplicateDest, volume.Dest)
+				return nil, nil, nil, fmt.Errorf("%v: %w", volume.Dest, specgen.ErrDuplicateDest)
 			}
 			finalNamedVolumes[volume.Dest] = volume
 		default:
-			return nil, nil, nil, errors.Errorf("invalid filesystem type %q", mountType)
+			return nil, nil, nil, fmt.Errorf("invalid filesystem type %q", mountType)
 		}
 	}
 
@@ -243,7 +262,7 @@ func getBindMount(args []string) (spec.Mount, error) {
 		Type: define.TypeBind,
 	}
 
-	var setSource, setDest, setRORW, setSuid, setDev, setExec, setRelabel bool
+	var setSource, setDest, setRORW, setSuid, setDev, setExec, setRelabel, setOwnership bool
 
 	for _, val := range args {
 		kv := strings.SplitN(val, "=", 2)
@@ -252,7 +271,7 @@ func getBindMount(args []string) (spec.Mount, error) {
 			newMount.Options = append(newMount.Options, "bind")
 		case "readonly", "ro", "rw":
 			if setRORW {
-				return newMount, errors.Wrapf(optionArgError, "cannot pass 'readonly', 'ro', or 'rw' options more than once")
+				return newMount, fmt.Errorf("cannot pass 'readonly', 'ro', or 'rw' options more than once: %w", errOptionArg)
 			}
 			setRORW = true
 			// Can be formatted as one of:
@@ -279,26 +298,26 @@ func getBindMount(args []string) (spec.Mount, error) {
 						newMount.Options = append(newMount.Options, "ro")
 					}
 				default:
-					return newMount, errors.Wrapf(optionArgError, "'readonly', 'ro', or 'rw' must be set to true or false, instead received %q", kv[1])
+					return newMount, fmt.Errorf("'readonly', 'ro', or 'rw' must be set to true or false, instead received %q: %w", kv[1], errOptionArg)
 				}
 			default:
-				return newMount, errors.Wrapf(optionArgError, "badly formatted option %q", val)
+				return newMount, fmt.Errorf("badly formatted option %q: %w", val, errOptionArg)
 			}
 		case "nosuid", "suid":
 			if setSuid {
-				return newMount, errors.Wrapf(optionArgError, "cannot pass 'nosuid' and 'suid' options more than once")
+				return newMount, fmt.Errorf("cannot pass 'nosuid' and 'suid' options more than once: %w", errOptionArg)
 			}
 			setSuid = true
 			newMount.Options = append(newMount.Options, kv[0])
 		case "nodev", "dev":
 			if setDev {
-				return newMount, errors.Wrapf(optionArgError, "cannot pass 'nodev' and 'dev' options more than once")
+				return newMount, fmt.Errorf("cannot pass 'nodev' and 'dev' options more than once: %w", errOptionArg)
 			}
 			setDev = true
 			newMount.Options = append(newMount.Options, kv[0])
 		case "noexec", "exec":
 			if setExec {
-				return newMount, errors.Wrapf(optionArgError, "cannot pass 'noexec' and 'exec' options more than once")
+				return newMount, fmt.Errorf("cannot pass 'noexec' and 'exec' options more than once: %w", errOptionArg)
 			}
 			setExec = true
 			newMount.Options = append(newMount.Options, kv[0])
@@ -306,34 +325,34 @@ func getBindMount(args []string) (spec.Mount, error) {
 			newMount.Options = append(newMount.Options, kv[0])
 		case "bind-propagation":
 			if len(kv) == 1 {
-				return newMount, errors.Wrapf(optionArgError, kv[0])
+				return newMount, fmt.Errorf("%v: %w", kv[0], errOptionArg)
 			}
 			newMount.Options = append(newMount.Options, kv[1])
 		case "src", "source":
 			if len(kv) == 1 {
-				return newMount, errors.Wrapf(optionArgError, kv[0])
+				return newMount, fmt.Errorf("%v: %w", kv[0], errOptionArg)
 			}
 			if len(kv[1]) == 0 {
-				return newMount, errors.Wrapf(optionArgError, "host directory cannot be empty")
+				return newMount, fmt.Errorf("host directory cannot be empty: %w", errOptionArg)
 			}
 			newMount.Source = kv[1]
 			setSource = true
 		case "target", "dst", "destination":
 			if len(kv) == 1 {
-				return newMount, errors.Wrapf(optionArgError, kv[0])
+				return newMount, fmt.Errorf("%v: %w", kv[0], errOptionArg)
 			}
 			if err := parse.ValidateVolumeCtrDir(kv[1]); err != nil {
 				return newMount, err
 			}
-			newMount.Destination = filepath.Clean(kv[1])
+			newMount.Destination = unixPathClean(kv[1])
 			setDest = true
 		case "relabel":
 			if setRelabel {
-				return newMount, errors.Wrapf(optionArgError, "cannot pass 'relabel' option more than once")
+				return newMount, fmt.Errorf("cannot pass 'relabel' option more than once: %w", errOptionArg)
 			}
 			setRelabel = true
 			if len(kv) != 2 {
-				return newMount, errors.Wrapf(util.ErrBadMntOption, "%s mount option must be 'private' or 'shared'", kv[0])
+				return newMount, fmt.Errorf("%s mount option must be 'private' or 'shared': %w", kv[0], util.ErrBadMntOption)
 			}
 			switch kv[1] {
 			case "private":
@@ -341,19 +360,37 @@ func getBindMount(args []string) (spec.Mount, error) {
 			case "shared":
 				newMount.Options = append(newMount.Options, "z")
 			default:
-				return newMount, errors.Wrapf(util.ErrBadMntOption, "%s mount option must be 'private' or 'shared'", kv[0])
+				return newMount, fmt.Errorf("%s mount option must be 'private' or 'shared': %w", kv[0], util.ErrBadMntOption)
+			}
+		case "U", "chown":
+			if setOwnership {
+				return newMount, fmt.Errorf("cannot pass 'U' or 'chown' option more than once: %w", errOptionArg)
+			}
+			ok, err := validChownFlag(val)
+			if err != nil {
+				return newMount, err
+			}
+			if ok {
+				newMount.Options = append(newMount.Options, "U")
+			}
+			setOwnership = true
+		case "idmap":
+			if len(kv) > 1 {
+				newMount.Options = append(newMount.Options, fmt.Sprintf("idmap=%s", kv[1]))
+			} else {
+				newMount.Options = append(newMount.Options, "idmap")
 			}
 		case "consistency":
 			// Often used on MACs and mistakenly on Linux platforms.
 			// Since Docker ignores this option so shall we.
 			continue
 		default:
-			return newMount, errors.Wrapf(util.ErrBadMntOption, kv[0])
+			return newMount, fmt.Errorf("%s: %w", kv[0], util.ErrBadMntOption)
 		}
 	}
 
 	if !setDest {
-		return newMount, noDestError
+		return newMount, errNoDest
 	}
 
 	if !setSource {
@@ -375,73 +412,85 @@ func getTmpfsMount(args []string) (spec.Mount, error) {
 		Source: define.TypeTmpfs,
 	}
 
-	var setDest, setRORW, setSuid, setDev, setExec, setTmpcopyup bool
+	var setDest, setRORW, setSuid, setDev, setExec, setTmpcopyup, setOwnership bool
 
 	for _, val := range args {
 		kv := strings.SplitN(val, "=", 2)
 		switch kv[0] {
 		case "tmpcopyup", "notmpcopyup":
 			if setTmpcopyup {
-				return newMount, errors.Wrapf(optionArgError, "cannot pass 'tmpcopyup' and 'notmpcopyup' options more than once")
+				return newMount, fmt.Errorf("cannot pass 'tmpcopyup' and 'notmpcopyup' options more than once: %w", errOptionArg)
 			}
 			setTmpcopyup = true
 			newMount.Options = append(newMount.Options, kv[0])
 		case "ro", "rw":
 			if setRORW {
-				return newMount, errors.Wrapf(optionArgError, "cannot pass 'ro' and 'rw' options more than once")
+				return newMount, fmt.Errorf("cannot pass 'ro' and 'rw' options more than once: %w", errOptionArg)
 			}
 			setRORW = true
 			newMount.Options = append(newMount.Options, kv[0])
 		case "nosuid", "suid":
 			if setSuid {
-				return newMount, errors.Wrapf(optionArgError, "cannot pass 'nosuid' and 'suid' options more than once")
+				return newMount, fmt.Errorf("cannot pass 'nosuid' and 'suid' options more than once: %w", errOptionArg)
 			}
 			setSuid = true
 			newMount.Options = append(newMount.Options, kv[0])
 		case "nodev", "dev":
 			if setDev {
-				return newMount, errors.Wrapf(optionArgError, "cannot pass 'nodev' and 'dev' options more than once")
+				return newMount, fmt.Errorf("cannot pass 'nodev' and 'dev' options more than once: %w", errOptionArg)
 			}
 			setDev = true
 			newMount.Options = append(newMount.Options, kv[0])
 		case "noexec", "exec":
 			if setExec {
-				return newMount, errors.Wrapf(optionArgError, "cannot pass 'noexec' and 'exec' options more than once")
+				return newMount, fmt.Errorf("cannot pass 'noexec' and 'exec' options more than once: %w", errOptionArg)
 			}
 			setExec = true
 			newMount.Options = append(newMount.Options, kv[0])
 		case "tmpfs-mode":
 			if len(kv) == 1 {
-				return newMount, errors.Wrapf(optionArgError, kv[0])
+				return newMount, fmt.Errorf("%v: %w", kv[0], errOptionArg)
 			}
 			newMount.Options = append(newMount.Options, fmt.Sprintf("mode=%s", kv[1]))
 		case "tmpfs-size":
 			if len(kv) == 1 {
-				return newMount, errors.Wrapf(optionArgError, kv[0])
+				return newMount, fmt.Errorf("%v: %w", kv[0], errOptionArg)
 			}
 			newMount.Options = append(newMount.Options, fmt.Sprintf("size=%s", kv[1]))
 		case "src", "source":
-			return newMount, errors.Errorf("source is not supported with tmpfs mounts")
+			return newMount, fmt.Errorf("source is not supported with tmpfs mounts")
 		case "target", "dst", "destination":
 			if len(kv) == 1 {
-				return newMount, errors.Wrapf(optionArgError, kv[0])
+				return newMount, fmt.Errorf("%v: %w", kv[0], errOptionArg)
 			}
 			if err := parse.ValidateVolumeCtrDir(kv[1]); err != nil {
 				return newMount, err
 			}
-			newMount.Destination = filepath.Clean(kv[1])
+			newMount.Destination = unixPathClean(kv[1])
 			setDest = true
+		case "U", "chown":
+			if setOwnership {
+				return newMount, fmt.Errorf("cannot pass 'U' or 'chown' option more than once: %w", errOptionArg)
+			}
+			ok, err := validChownFlag(val)
+			if err != nil {
+				return newMount, err
+			}
+			if ok {
+				newMount.Options = append(newMount.Options, "U")
+			}
+			setOwnership = true
 		case "consistency":
 			// Often used on MACs and mistakenly on Linux platforms.
 			// Since Docker ignores this option so shall we.
 			continue
 		default:
-			return newMount, errors.Wrapf(util.ErrBadMntOption, kv[0])
+			return newMount, fmt.Errorf("%s: %w", kv[0], util.ErrBadMntOption)
 		}
 	}
 
 	if !setDest {
-		return newMount, noDestError
+		return newMount, errNoDest
 	}
 
 	return newMount, nil
@@ -459,22 +508,24 @@ func getDevptsMount(args []string) (spec.Mount, error) {
 	for _, val := range args {
 		kv := strings.SplitN(val, "=", 2)
 		switch kv[0] {
+		case "uid", "gid", "mode", "ptxmode", "newinstance", "max":
+			newMount.Options = append(newMount.Options, val)
 		case "target", "dst", "destination":
 			if len(kv) == 1 {
-				return newMount, errors.Wrapf(optionArgError, kv[0])
+				return newMount, fmt.Errorf("%v: %w", kv[0], errOptionArg)
 			}
 			if err := parse.ValidateVolumeCtrDir(kv[1]); err != nil {
 				return newMount, err
 			}
-			newMount.Destination = filepath.Clean(kv[1])
+			newMount.Destination = unixPathClean(kv[1])
 			setDest = true
 		default:
-			return newMount, errors.Wrapf(util.ErrBadMntOption, kv[0])
+			return newMount, fmt.Errorf("%s: %w", kv[0], util.ErrBadMntOption)
 		}
 	}
 
 	if !setDest {
-		return newMount, noDestError
+		return newMount, errNoDest
 	}
 
 	return newMount, nil
@@ -486,66 +537,76 @@ func getDevptsMount(args []string) (spec.Mount, error) {
 func getNamedVolume(args []string) (*specgen.NamedVolume, error) {
 	newVolume := new(specgen.NamedVolume)
 
-	var setSource, setDest, setRORW, setSuid, setDev, setExec bool
+	var setDest, setRORW, setSuid, setDev, setExec, setOwnership bool
 
 	for _, val := range args {
 		kv := strings.SplitN(val, "=", 2)
 		switch kv[0] {
+		case "volume-opt":
+			newVolume.Options = append(newVolume.Options, val)
 		case "ro", "rw":
 			if setRORW {
-				return nil, errors.Wrapf(optionArgError, "cannot pass 'ro' and 'rw' options more than once")
+				return nil, fmt.Errorf("cannot pass 'ro' and 'rw' options more than once: %w", errOptionArg)
 			}
 			setRORW = true
 			newVolume.Options = append(newVolume.Options, kv[0])
 		case "nosuid", "suid":
 			if setSuid {
-				return nil, errors.Wrapf(optionArgError, "cannot pass 'nosuid' and 'suid' options more than once")
+				return nil, fmt.Errorf("cannot pass 'nosuid' and 'suid' options more than once: %w", errOptionArg)
 			}
 			setSuid = true
 			newVolume.Options = append(newVolume.Options, kv[0])
 		case "nodev", "dev":
 			if setDev {
-				return nil, errors.Wrapf(optionArgError, "cannot pass 'nodev' and 'dev' options more than once")
+				return nil, fmt.Errorf("cannot pass 'nodev' and 'dev' options more than once: %w", errOptionArg)
 			}
 			setDev = true
 			newVolume.Options = append(newVolume.Options, kv[0])
 		case "noexec", "exec":
 			if setExec {
-				return nil, errors.Wrapf(optionArgError, "cannot pass 'noexec' and 'exec' options more than once")
+				return nil, fmt.Errorf("cannot pass 'noexec' and 'exec' options more than once: %w", errOptionArg)
 			}
 			setExec = true
 			newVolume.Options = append(newVolume.Options, kv[0])
 		case "volume-label":
-			return nil, errors.Errorf("the --volume-label option is not presently implemented")
+			return nil, fmt.Errorf("the --volume-label option is not presently implemented")
 		case "src", "source":
 			if len(kv) == 1 {
-				return nil, errors.Wrapf(optionArgError, kv[0])
+				return nil, fmt.Errorf("%v: %w", kv[0], errOptionArg)
 			}
 			newVolume.Name = kv[1]
-			setSource = true
 		case "target", "dst", "destination":
 			if len(kv) == 1 {
-				return nil, errors.Wrapf(optionArgError, kv[0])
+				return nil, fmt.Errorf("%v: %w", kv[0], errOptionArg)
 			}
 			if err := parse.ValidateVolumeCtrDir(kv[1]); err != nil {
 				return nil, err
 			}
-			newVolume.Dest = filepath.Clean(kv[1])
+			newVolume.Dest = unixPathClean(kv[1])
 			setDest = true
+		case "U", "chown":
+			if setOwnership {
+				return newVolume, fmt.Errorf("cannot pass 'U' or 'chown' option more than once: %w", errOptionArg)
+			}
+			ok, err := validChownFlag(val)
+			if err != nil {
+				return newVolume, err
+			}
+			if ok {
+				newVolume.Options = append(newVolume.Options, "U")
+			}
+			setOwnership = true
 		case "consistency":
 			// Often used on MACs and mistakenly on Linux platforms.
 			// Since Docker ignores this option so shall we.
 			continue
 		default:
-			return nil, errors.Wrapf(util.ErrBadMntOption, kv[0])
+			return nil, fmt.Errorf("%s: %w", kv[0], util.ErrBadMntOption)
 		}
 	}
 
-	if !setSource {
-		return nil, errors.Errorf("must set source volume")
-	}
 	if !setDest {
-		return nil, noDestError
+		return nil, errNoDest
 	}
 
 	return newVolume, nil
@@ -554,7 +615,7 @@ func getNamedVolume(args []string) (*specgen.NamedVolume, error) {
 // Parse the arguments into an image volume. An image volume is a volume based
 // on a container image.  The container image is first mounted on the host and
 // is then bind-mounted into the container.  An ImageVolume is always mounted
-// read only.
+// read-only.
 func getImageVolume(args []string) (*specgen.ImageVolume, error) {
 	newVolume := new(specgen.ImageVolume)
 
@@ -563,17 +624,17 @@ func getImageVolume(args []string) (*specgen.ImageVolume, error) {
 		switch kv[0] {
 		case "src", "source":
 			if len(kv) == 1 {
-				return nil, errors.Wrapf(optionArgError, kv[0])
+				return nil, fmt.Errorf("%v: %w", kv[0], errOptionArg)
 			}
 			newVolume.Source = kv[1]
 		case "target", "dst", "destination":
 			if len(kv) == 1 {
-				return nil, errors.Wrapf(optionArgError, kv[0])
+				return nil, fmt.Errorf("%v: %w", kv[0], errOptionArg)
 			}
 			if err := parse.ValidateVolumeCtrDir(kv[1]); err != nil {
 				return nil, err
 			}
-			newVolume.Destination = filepath.Clean(kv[1])
+			newVolume.Destination = unixPathClean(kv[1])
 		case "rw", "readwrite":
 			switch kv[1] {
 			case "true":
@@ -581,19 +642,19 @@ func getImageVolume(args []string) (*specgen.ImageVolume, error) {
 			case "false":
 				// Nothing to do. RO is default.
 			default:
-				return nil, errors.Wrapf(util.ErrBadMntOption, "invalid rw value %q", kv[1])
+				return nil, fmt.Errorf("invalid rw value %q: %w", kv[1], util.ErrBadMntOption)
 			}
 		case "consistency":
 			// Often used on MACs and mistakenly on Linux platforms.
 			// Since Docker ignores this option so shall we.
 			continue
 		default:
-			return nil, errors.Wrapf(util.ErrBadMntOption, kv[0])
+			return nil, fmt.Errorf("%s: %w", kv[0], util.ErrBadMntOption)
 		}
 	}
 
 	if len(newVolume.Source)*len(newVolume.Destination) == 0 {
-		return nil, errors.Errorf("must set source and destination for image volume")
+		return nil, errors.New("must set source and destination for image volume")
 	}
 
 	return newVolume, nil
@@ -614,12 +675,14 @@ func getTmpfsMounts(tmpfsFlag []string) (map[string]spec.Mount, error) {
 			options = strings.Split(spliti[1], ",")
 		}
 
-		if _, ok := m[destPath]; ok {
-			return nil, errors.Wrapf(errDuplicateDest, destPath)
+		if vol, ok := m[destPath]; ok {
+			if specgen.StringSlicesEqual(vol.Options, options) {
+				continue
+			}
+			return nil, fmt.Errorf("%v: %w", destPath, specgen.ErrDuplicateDest)
 		}
-
 		mount := spec.Mount{
-			Destination: filepath.Clean(destPath),
+			Destination: unixPathClean(destPath),
 			Type:        define.TypeTmpfs,
 			Options:     options,
 			Source:      define.TypeTmpfs,
@@ -628,3 +691,29 @@ func getTmpfsMounts(tmpfsFlag []string) (map[string]spec.Mount, error) {
 	}
 	return m, nil
 }
+
+// validChownFlag ensures that the U or chown flag is correctly used
+func validChownFlag(flag string) (bool, error) {
+	kv := strings.SplitN(flag, "=", 2)
+	switch len(kv) {
+	case 1:
+	case 2:
+		// U=[true|false]
+		switch strings.ToLower(kv[1]) {
+		case "true":
+		case "false":
+			return false, nil
+		default:
+			return false, fmt.Errorf("'U' or 'chown' must be set to true or false, instead received %q: %w", kv[1], errOptionArg)
+		}
+	default:
+		return false, fmt.Errorf("badly formatted option %q: %w", flag, errOptionArg)
+	}
+
+	return true, nil
+}
+
+// Use path instead of filepath to preserve Unix style paths on Windows
+func unixPathClean(p string) string {
+	return path.Clean(p)
+}
