@@ -1,21 +1,21 @@
 package integration
 
 import (
-	"io/ioutil"
 	"os"
+	"os/user"
 	"path/filepath"
 	"strconv"
 	"strings"
 
-	"github.com/containers/podman/v3/libpod/define"
+	"github.com/containers/podman/v4/libpod/define"
 
-	"github.com/containers/podman/v3/pkg/util"
-	. "github.com/containers/podman/v3/test/utils"
+	v1 "github.com/containers/podman/v4/pkg/k8s.io/api/core/v1"
+	"github.com/containers/podman/v4/pkg/util"
+	. "github.com/containers/podman/v4/test/utils"
 	"github.com/ghodss/yaml"
 	. "github.com/onsi/ginkgo"
 	. "github.com/onsi/gomega"
 	. "github.com/onsi/gomega/gexec"
-	v1 "k8s.io/api/core/v1"
 )
 
 var _ = Describe("Podman generate kube", func() {
@@ -32,7 +32,6 @@ var _ = Describe("Podman generate kube", func() {
 		}
 		podmanTest = PodmanTestCreate(tempdir)
 		podmanTest.Setup()
-		podmanTest.SeedImages()
 	})
 
 	AfterEach(func() {
@@ -59,22 +58,27 @@ var _ = Describe("Podman generate kube", func() {
 		session.WaitWithDefaultTimeout()
 		Expect(session).Should(Exit(0))
 
-		kube := podmanTest.Podman([]string{"generate", "kube", "top"})
+		kube := podmanTest.Podman([]string{"kube", "generate", "top"})
 		kube.WaitWithDefaultTimeout()
 		Expect(kube).Should(Exit(0))
 
 		pod := new(v1.Pod)
 		err := yaml.Unmarshal(kube.Out.Contents(), pod)
 		Expect(err).To(BeNil())
-		Expect(pod.Spec.HostNetwork).To(Equal(false))
+		Expect(pod.Spec).To(HaveField("HostNetwork", false))
 		Expect(pod.Spec.SecurityContext).To(BeNil())
 		Expect(pod.Spec.DNSConfig).To(BeNil())
-		Expect(pod.Spec.Containers[0].WorkingDir).To(Equal(""))
+		Expect(pod.Spec.Containers[0]).To(HaveField("WorkingDir", ""))
 		Expect(pod.Spec.Containers[0].Env).To(BeNil())
+		Expect(pod).To(HaveField("Name", "top-pod"))
+		enableServiceLinks := false
+		Expect(pod.Spec).To(HaveField("EnableServiceLinks", &enableServiceLinks))
+		automountServiceAccountToken := false
+		Expect(pod.Spec).To(HaveField("AutomountServiceAccountToken", &automountServiceAccountToken))
 
 		numContainers := 0
 		for range pod.Spec.Containers {
-			numContainers = numContainers + 1
+			numContainers++
 		}
 		Expect(numContainers).To(Equal(1))
 	})
@@ -115,7 +119,7 @@ var _ = Describe("Podman generate kube", func() {
 		session.WaitWithDefaultTimeout()
 		Expect(session).Should(Exit(0))
 
-		kube := podmanTest.Podman([]string{"generate", "kube", "test"})
+		kube := podmanTest.Podman([]string{"kube", "generate", "test"})
 		kube.WaitWithDefaultTimeout()
 		Expect(kube).Should(Exit(0))
 
@@ -130,18 +134,18 @@ var _ = Describe("Podman generate kube", func() {
 		session.WaitWithDefaultTimeout()
 		Expect(session).Should(Exit(0))
 
-		kube := podmanTest.Podman([]string{"generate", "kube", "-s", "test-ctr"})
+		kube := podmanTest.Podman([]string{"kube", "generate", "-s", "test-ctr"})
 		kube.WaitWithDefaultTimeout()
 		Expect(kube).Should(Exit(0))
 
 		// Separate out the Service and Pod yaml
 		arr := strings.Split(string(kube.Out.Contents()), "---")
-		Expect(len(arr)).To(Equal(2))
+		Expect(arr).To(HaveLen(2))
 
 		svc := new(v1.Service)
 		err := yaml.Unmarshal([]byte(arr[0]), svc)
 		Expect(err).To(BeNil())
-		Expect(len(svc.Spec.Ports)).To(Equal(1))
+		Expect(svc.Spec.Ports).To(HaveLen(1))
 		Expect(svc.Spec.Ports[0].TargetPort.IntValue()).To(Equal(3890))
 
 		pod := new(v1.Pod)
@@ -164,11 +168,15 @@ var _ = Describe("Podman generate kube", func() {
 		pod := new(v1.Pod)
 		err := yaml.Unmarshal(kube.Out.Contents(), pod)
 		Expect(err).To(BeNil())
-		Expect(pod.Spec.HostNetwork).To(Equal(false))
+		Expect(pod.Spec).To(HaveField("HostNetwork", false))
+		enableServiceLinks := false
+		Expect(pod.Spec).To(HaveField("EnableServiceLinks", &enableServiceLinks))
+		automountServiceAccountToken := false
+		Expect(pod.Spec).To(HaveField("AutomountServiceAccountToken", &automountServiceAccountToken))
 
 		numContainers := 0
 		for range pod.Spec.Containers {
-			numContainers = numContainers + 1
+			numContainers++
 		}
 		Expect(numContainers).To(Equal(1))
 	})
@@ -182,7 +190,7 @@ var _ = Describe("Podman generate kube", func() {
 		pod2.WaitWithDefaultTimeout()
 		Expect(pod2).Should(Exit(0))
 
-		kube := podmanTest.Podman([]string{"generate", "kube", "pod1", "pod2"})
+		kube := podmanTest.Podman([]string{"kube", "generate", "pod1", "pod2"})
 		kube.WaitWithDefaultTimeout()
 		Expect(kube).Should(Exit(0))
 
@@ -203,14 +211,14 @@ var _ = Describe("Podman generate kube", func() {
 		session.WaitWithDefaultTimeout()
 		Expect(session).Should(Exit(0))
 
-		kube := podmanTest.Podman([]string{"generate", "kube", "toppod"})
+		kube := podmanTest.Podman([]string{"kube", "generate", "toppod"})
 		kube.WaitWithDefaultTimeout()
 		Expect(kube).Should(Exit(0))
 
 		pod := new(v1.Pod)
 		err := yaml.Unmarshal(kube.Out.Contents(), pod)
 		Expect(err).To(BeNil())
-		Expect(pod.Spec.HostNetwork).To(Equal(false))
+		Expect(pod.Spec).To(HaveField("HostNetwork", false))
 
 		numContainers := len(pod.Spec.Containers) + len(pod.Spec.InitContainers)
 		Expect(numContainers).To(Equal(3))
@@ -231,7 +239,7 @@ var _ = Describe("Podman generate kube", func() {
 		pod = new(v1.Pod)
 		err = yaml.Unmarshal(kube.Out.Contents(), pod)
 		Expect(err).To(BeNil())
-		Expect(pod.Spec.HostNetwork).To(Equal(false))
+		Expect(pod.Spec).To(HaveField("HostNetwork", false))
 
 		numContainers = len(pod.Spec.Containers) + len(pod.Spec.InitContainers)
 		Expect(numContainers).To(Equal(2))
@@ -256,12 +264,45 @@ var _ = Describe("Podman generate kube", func() {
 		pod = new(v1.Pod)
 		err = yaml.Unmarshal(kube.Out.Contents(), pod)
 		Expect(err).To(BeNil())
-		Expect(pod.Spec.HostNetwork).To(Equal(false))
+		Expect(pod.Spec).To(HaveField("HostNetwork", false))
 
 		numContainers = len(pod.Spec.Containers) + len(pod.Spec.InitContainers)
 		Expect(numContainers).To(Equal(1))
 	})
 
+	It("podman generate kube on pod with user namespace", func() {
+		u, err := user.Current()
+		Expect(err).To(BeNil())
+		name := u.Name
+		if name == "root" {
+			name = "containers"
+		}
+		content, err := os.ReadFile("/etc/subuid")
+		if err != nil {
+			Skip("cannot read /etc/subuid")
+		}
+		if !strings.Contains(string(content), name) {
+			Skip("cannot find mappings for the current user")
+		}
+		podSession := podmanTest.Podman([]string{"pod", "create", "--name", "testPod", "--userns=auto"})
+		podSession.WaitWithDefaultTimeout()
+		Expect(podSession).Should(Exit(0))
+
+		session := podmanTest.Podman([]string{"create", "--name", "topcontainer", "--pod", "testPod", ALPINE, "top"})
+		session.WaitWithDefaultTimeout()
+		Expect(session).Should(Exit(0))
+
+		kube := podmanTest.Podman([]string{"generate", "kube", "testPod"})
+		kube.WaitWithDefaultTimeout()
+		Expect(kube).Should(Exit(0))
+
+		pod := new(v1.Pod)
+		err = yaml.Unmarshal(kube.Out.Contents(), pod)
+		Expect(err).To(BeNil())
+		expected := false
+		Expect(pod.Spec).To(HaveField("HostUsers", &expected))
+	})
+
 	It("podman generate kube on pod with host network", func() {
 		podSession := podmanTest.Podman([]string{"pod", "create", "--name", "testHostNetwork", "--network", "host"})
 		podSession.WaitWithDefaultTimeout()
@@ -278,7 +319,7 @@ var _ = Describe("Podman generate kube", func() {
 		pod := new(v1.Pod)
 		err := yaml.Unmarshal(kube.Out.Contents(), pod)
 		Expect(err).To(BeNil())
-		Expect(pod.Spec.HostNetwork).To(Equal(true))
+		Expect(pod.Spec).To(HaveField("HostNetwork", true))
 	})
 
 	It("podman generate kube on container with host network", func() {
@@ -293,7 +334,7 @@ var _ = Describe("Podman generate kube", func() {
 		pod := new(v1.Pod)
 		err := yaml.Unmarshal(kube.Out.Contents(), pod)
 		Expect(err).To(BeNil())
-		Expect(pod.Spec.HostNetwork).To(Equal(true))
+		Expect(pod.Spec).To(HaveField("HostNetwork", true))
 	})
 
 	It("podman generate kube on pod with hostAliases", func() {
@@ -323,9 +364,112 @@ var _ = Describe("Podman generate kube", func() {
 		pod := new(v1.Pod)
 		err := yaml.Unmarshal(kube.Out.Contents(), pod)
 		Expect(err).To(BeNil())
-		Expect(len(pod.Spec.HostAliases)).To(Equal(2))
-		Expect(pod.Spec.HostAliases[0].IP).To(Equal(testIP))
-		Expect(pod.Spec.HostAliases[1].IP).To(Equal(testIP))
+		Expect(pod.Spec.HostAliases).To(HaveLen(2))
+		Expect(pod.Spec.HostAliases[0]).To(HaveField("IP", testIP))
+		Expect(pod.Spec.HostAliases[1]).To(HaveField("IP", testIP))
+	})
+
+	It("podman generate kube with network sharing", func() {
+		// Expect error with default sharing options as Net namespace is shared
+		podName := "testPod"
+		podSession := podmanTest.Podman([]string{"pod", "create", "--name", podName})
+		podSession.WaitWithDefaultTimeout()
+		Expect(podSession).Should(Exit(0))
+
+		ctrSession := podmanTest.Podman([]string{"create", "--name", "testCtr", "--pod", podName, "-p", "9000:8000", ALPINE, "top"})
+		ctrSession.WaitWithDefaultTimeout()
+		Expect(ctrSession).Should(Exit(125))
+
+		// Ports without Net sharing should work with ports being set for each container in the generated kube yaml
+		podName = "testNet"
+		podSession = podmanTest.Podman([]string{"pod", "create", "--name", podName, "--share", "ipc"})
+		podSession.WaitWithDefaultTimeout()
+		Expect(podSession).Should(Exit(0))
+
+		ctr1Name := "ctr1"
+		ctr1Session := podmanTest.Podman([]string{"create", "--name", ctr1Name, "--pod", podName, "-p", "9000:8000", ALPINE, "top"})
+		ctr1Session.WaitWithDefaultTimeout()
+		Expect(ctr1Session).Should(Exit(0))
+
+		ctr2Name := "ctr2"
+		ctr2Session := podmanTest.Podman([]string{"create", "--name", ctr2Name, "--pod", podName, "-p", "6000:5000", ALPINE, "top"})
+		ctr2Session.WaitWithDefaultTimeout()
+		Expect(ctr2Session).Should(Exit(0))
+
+		kube := podmanTest.Podman([]string{"generate", "kube", podName})
+		kube.WaitWithDefaultTimeout()
+		Expect(kube).Should(Exit(0))
+
+		pod := new(v1.Pod)
+		err := yaml.Unmarshal(kube.Out.Contents(), pod)
+		Expect(err).To(BeNil())
+		Expect(pod.Spec.Containers).To(HaveLen(2))
+		Expect(pod.Spec.Containers[0].Ports[0].ContainerPort).To(Equal(int32(8000)))
+		Expect(pod.Spec.Containers[1].Ports[0].ContainerPort).To(Equal(int32(5000)))
+		Expect(pod.Spec.Containers[0].Ports[0].HostPort).To(Equal(int32(9000)))
+		Expect(pod.Spec.Containers[1].Ports[0].HostPort).To(Equal(int32(6000)))
+	})
+
+	It("podman generate kube with and without hostname", func() {
+		// Expect error with default sharing options as UTS namespace is shared
+		podName := "testPod"
+		podSession := podmanTest.Podman([]string{"pod", "create", "--name", podName})
+		podSession.WaitWithDefaultTimeout()
+		Expect(podSession).Should(Exit(0))
+
+		ctrSession := podmanTest.Podman([]string{"create", "--name", "testCtr", "--pod", podName, "--hostname", "test-hostname", ALPINE, "top"})
+		ctrSession.WaitWithDefaultTimeout()
+		Expect(ctrSession).Should(Exit(125))
+
+		// Hostname without uts sharing should work, but generated kube yaml will have pod hostname
+		// set to the hostname of the first container
+		podName = "testHostname"
+		podSession = podmanTest.Podman([]string{"pod", "create", "--name", podName, "--share", "ipc"})
+		podSession.WaitWithDefaultTimeout()
+		Expect(podSession).Should(Exit(0))
+
+		ctr1Name := "ctr1"
+		ctr1HostName := "ctr1-hostname"
+		ctr1Session := podmanTest.Podman([]string{"create", "--name", ctr1Name, "--pod", podName, "--hostname", ctr1HostName, ALPINE, "top"})
+		ctr1Session.WaitWithDefaultTimeout()
+		Expect(ctr1Session).Should(Exit(0))
+
+		ctr2Name := "ctr2"
+		ctr2Session := podmanTest.Podman([]string{"create", "--name", ctr2Name, "--pod", podName, ALPINE, "top"})
+		ctr2Session.WaitWithDefaultTimeout()
+		Expect(ctr2Session).Should(Exit(0))
+
+		kube := podmanTest.Podman([]string{"kube", "generate", podName})
+		kube.WaitWithDefaultTimeout()
+		Expect(kube).Should(Exit(0))
+
+		pod := new(v1.Pod)
+		err := yaml.Unmarshal(kube.Out.Contents(), pod)
+		Expect(err).To(BeNil())
+		Expect(pod.Spec.Containers).To(HaveLen(2))
+		Expect(pod.Spec.Hostname).To(Equal(ctr1HostName))
+
+		// No hostname
+
+		podName = "testNoHostname"
+		podSession = podmanTest.Podman([]string{"pod", "create", "--name", podName, "--share", "ipc"})
+		podSession.WaitWithDefaultTimeout()
+		Expect(podSession).Should(Exit(0))
+
+		ctr3Name := "ctr3"
+		ctr3Session := podmanTest.Podman([]string{"create", "--name", ctr3Name, "--pod", podName, ALPINE, "top"})
+		ctr3Session.WaitWithDefaultTimeout()
+		Expect(ctr3Session).Should(Exit(0))
+
+		kube = podmanTest.Podman([]string{"generate", "kube", podName})
+		kube.WaitWithDefaultTimeout()
+		Expect(kube).Should(Exit(0))
+
+		pod = new(v1.Pod)
+		err = yaml.Unmarshal(kube.Out.Contents(), pod)
+		Expect(err).To(BeNil())
+		Expect(pod.Spec.Containers).To(HaveLen(1))
+		Expect(pod.Spec.Hostname).To(BeEmpty())
 	})
 
 	It("podman generate service kube on pod", func() {
@@ -339,14 +483,14 @@ var _ = Describe("Podman generate kube", func() {
 
 		// Separate out the Service and Pod yaml
 		arr := strings.Split(string(kube.Out.Contents()), "---")
-		Expect(len(arr)).To(Equal(2))
+		Expect(arr).To(HaveLen(2))
 
 		svc := new(v1.Service)
 		err := yaml.Unmarshal([]byte(arr[0]), svc)
 		Expect(err).To(BeNil())
-		Expect(len(svc.Spec.Ports)).To(Equal(1))
+		Expect(svc.Spec.Ports).To(HaveLen(1))
 		Expect(svc.Spec.Ports[0].TargetPort.IntValue()).To(Equal(4000))
-		Expect(svc.Spec.Ports[0].Protocol).To(Equal(v1.ProtocolUDP))
+		Expect(svc.Spec.Ports[0]).To(HaveField("Protocol", v1.ProtocolUDP))
 
 		pod := new(v1.Pod)
 		err = yaml.Unmarshal([]byte(arr[1]), pod)
@@ -387,17 +531,18 @@ var _ = Describe("Podman generate kube", func() {
 	})
 
 	It("podman generate kube on pod with memory limit", func() {
+		SkipIfRootlessCgroupsV1("Not supported for rootless + CgroupsV1")
 		podName := "testMemoryLimit"
 		podSession := podmanTest.Podman([]string{"pod", "create", "--name", podName})
 		podSession.WaitWithDefaultTimeout()
 		Expect(podSession).Should(Exit(0))
 
 		ctr1Name := "ctr1"
-		ctr1Session := podmanTest.Podman([]string{"create", "--name", ctr1Name, "--pod", podName, "--memory", "10Mi", ALPINE, "top"})
+		ctr1Session := podmanTest.Podman([]string{"create", "--name", ctr1Name, "--pod", podName, "--memory", "10M", ALPINE, "top"})
 		ctr1Session.WaitWithDefaultTimeout()
 		Expect(ctr1Session).Should(Exit(0))
 
-		kube := podmanTest.Podman([]string{"generate", "kube", podName})
+		kube := podmanTest.Podman([]string{"kube", "generate", podName})
 		kube.WaitWithDefaultTimeout()
 		Expect(kube).Should(Exit(0))
 
@@ -412,6 +557,7 @@ var _ = Describe("Podman generate kube", func() {
 	})
 
 	It("podman generate kube on pod with cpu limit", func() {
+		SkipIfRootlessCgroupsV1("Not supported for rootless + CgroupsV1")
 		podName := "testCpuLimit"
 		podSession := podmanTest.Podman([]string{"pod", "create", "--name", podName})
 		podSession.WaitWithDefaultTimeout()
@@ -445,6 +591,11 @@ var _ = Describe("Podman generate kube", func() {
 
 	It("podman generate kube on pod with ports", func() {
 		podName := "test"
+
+		lock4 := GetPortLock("4000")
+		defer lock4.Unlock()
+		lock5 := GetPortLock("5000")
+		defer lock5.Unlock()
 		podSession := podmanTest.Podman([]string{"pod", "create", "--name", podName, "-p", "4000:4000", "-p", "5000:5000"})
 		podSession.WaitWithDefaultTimeout()
 		Expect(podSession).Should(Exit(0))
@@ -459,7 +610,7 @@ var _ = Describe("Podman generate kube", func() {
 		ctr2Session.WaitWithDefaultTimeout()
 		Expect(ctr2Session).Should(Exit(0))
 
-		kube := podmanTest.Podman([]string{"generate", "kube", podName})
+		kube := podmanTest.Podman([]string{"kube", "generate", podName})
 		kube.WaitWithDefaultTimeout()
 		Expect(kube).Should(Exit(0))
 
@@ -477,11 +628,11 @@ var _ = Describe("Podman generate kube", func() {
 				// for k8s
 				Expect(port.Protocol).To(BeEmpty())
 				if port.HostPort == 4000 {
-					foundPort4000 = foundPort4000 + 1
+					foundPort4000++
 				} else if port.HostPort == 5000 {
-					foundPort5000 = foundPort5000 + 1
+					foundPort5000++
 				} else {
-					foundOtherPort = foundOtherPort + 1
+					foundOtherPort++
 				}
 			}
 		}
@@ -503,9 +654,9 @@ var _ = Describe("Podman generate kube", func() {
 		Expect(err).To(BeNil())
 
 		containers := pod.Spec.Containers
-		Expect(len(containers)).To(Equal(1))
-		Expect(len(containers[0].Ports)).To(Equal(1))
-		Expect(containers[0].Ports[0].Protocol).To(Equal(v1.ProtocolUDP))
+		Expect(containers).To(HaveLen(1))
+		Expect(containers[0].Ports).To(HaveLen(1))
+		Expect(containers[0].Ports[0]).To(HaveField("Protocol", v1.ProtocolUDP))
 	})
 
 	It("podman generate and reimport kube on pod", func() {
@@ -562,7 +713,7 @@ var _ = Describe("Podman generate kube", func() {
 		Expect(inspect.OutputToString()).To(ContainSubstring("100:200"))
 
 		outputFile := filepath.Join(podmanTest.RunRoot, "pod.yaml")
-		kube := podmanTest.Podman([]string{"generate", "kube", "-f", outputFile, podName})
+		kube := podmanTest.Podman([]string{"kube", "generate", "-f", outputFile, podName})
 		kube.WaitWithDefaultTimeout()
 		Expect(kube).Should(Exit(0))
 
@@ -600,16 +751,14 @@ var _ = Describe("Podman generate kube", func() {
 		kube.WaitWithDefaultTimeout()
 		Expect(kube).Should(Exit(0))
 
-		b, err := ioutil.ReadFile(outputFile)
+		b, err := os.ReadFile(outputFile)
 		Expect(err).ShouldNot(HaveOccurred())
 		pod := new(v1.Pod)
 		err = yaml.Unmarshal(b, pod)
 		Expect(err).To(BeNil())
-		val, found := pod.Annotations[define.BindMountPrefix+vol1]
-		Expect(found).To(BeTrue())
-		Expect(val).To(HaveSuffix("z"))
+		Expect(pod.Annotations).To(HaveKeyWithValue(define.BindMountPrefix, vol1+":"+"z"))
 
-		rm := podmanTest.Podman([]string{"pod", "rm", "-f", "test1"})
+		rm := podmanTest.Podman([]string{"pod", "rm", "-t", "0", "-f", "test1"})
 		rm.WaitWithDefaultTimeout()
 		Expect(rm).Should(Exit(0))
 
@@ -642,7 +791,7 @@ var _ = Describe("Podman generate kube", func() {
 		err := yaml.Unmarshal(kube.Out.Contents(), pod)
 		Expect(err).To(BeNil())
 
-		Expect(len(pod.Spec.Volumes)).To(Equal(2))
+		Expect(pod.Spec.Volumes).To(HaveLen(2))
 
 	})
 
@@ -662,7 +811,7 @@ var _ = Describe("Podman generate kube", func() {
 		kube.WaitWithDefaultTimeout()
 		Expect(kube).Should(Exit(0))
 
-		rm := podmanTest.Podman([]string{"pod", "rm", "-f", "test1"})
+		rm := podmanTest.Podman([]string{"pod", "rm", "-t", "0", "-f", "test1"})
 		rm.WaitWithDefaultTimeout()
 		Expect(rm).Should(Exit(0))
 
@@ -687,11 +836,11 @@ var _ = Describe("Podman generate kube", func() {
 		Expect(session).Should(Exit(0))
 
 		outputFile := filepath.Join(podmanTest.RunRoot, "pod.yaml")
-		kube := podmanTest.Podman([]string{"generate", "kube", podName, "-f", outputFile})
+		kube := podmanTest.Podman([]string{"kube", "generate", podName, "-f", outputFile})
 		kube.WaitWithDefaultTimeout()
 		Expect(kube).Should(Exit(0))
 
-		rm := podmanTest.Podman([]string{"pod", "rm", "-f", podName})
+		rm := podmanTest.Podman([]string{"pod", "rm", "-t", "0", "-f", podName})
 		rm.WaitWithDefaultTimeout()
 		Expect(rm).Should(Exit(0))
 
@@ -756,13 +905,13 @@ var _ = Describe("Podman generate kube", func() {
 		pod2.WaitWithDefaultTimeout()
 		Expect(pod2).Should(Exit(0))
 
-		kube := podmanTest.Podman([]string{"generate", "kube", "top1", "top2"})
+		kube := podmanTest.Podman([]string{"kube", "generate", "top1", "top2"})
 		kube.WaitWithDefaultTimeout()
 		Expect(kube).To(ExitWithError())
 	})
 
 	It("podman generate kube on a container with dns options", func() {
-		top := podmanTest.Podman([]string{"run", "-dt", "--name", "top", "--dns", "8.8.8.8", "--dns-search", "foobar.com", "--dns-opt", "color:blue", ALPINE, "top"})
+		top := podmanTest.Podman([]string{"run", "-dt", "--name", "top", "--dns", "8.8.8.8", "--dns-search", "foobar.com", "--dns-option", "color:blue", ALPINE, "top"})
 		top.WaitWithDefaultTimeout()
 		Expect(top).Should(Exit(0))
 
@@ -774,11 +923,12 @@ var _ = Describe("Podman generate kube", func() {
 		err := yaml.Unmarshal(kube.Out.Contents(), pod)
 		Expect(err).To(BeNil())
 
-		Expect(StringInSlice("8.8.8.8", pod.Spec.DNSConfig.Nameservers)).To(BeTrue())
-		Expect(StringInSlice("foobar.com", pod.Spec.DNSConfig.Searches)).To(BeTrue())
+		Expect(pod.Spec.DNSConfig.Nameservers).To(ContainElement("8.8.8.8"))
+		Expect(pod.Spec.DNSConfig.Searches).To(ContainElement("foobar.com"))
 		Expect(len(pod.Spec.DNSConfig.Options)).To(BeNumerically(">", 0))
-		Expect(pod.Spec.DNSConfig.Options[0].Name).To(Equal("color"))
-		Expect(*pod.Spec.DNSConfig.Options[0].Value).To(Equal("blue"))
+		Expect(pod.Spec.DNSConfig.Options[0]).To(HaveField("Name", "color"))
+		s := "blue"
+		Expect(pod.Spec.DNSConfig.Options[0]).To(HaveField("Value", &s))
 	})
 
 	It("podman generate kube multiple container dns servers and options are cumulative", func() {
@@ -798,10 +948,10 @@ var _ = Describe("Podman generate kube", func() {
 		err := yaml.Unmarshal(kube.Out.Contents(), pod)
 		Expect(err).To(BeNil())
 
-		Expect(StringInSlice("8.8.8.8", pod.Spec.DNSConfig.Nameservers)).To(BeTrue())
-		Expect(StringInSlice("8.7.7.7", pod.Spec.DNSConfig.Nameservers)).To(BeTrue())
-		Expect(StringInSlice("foobar.com", pod.Spec.DNSConfig.Searches)).To(BeTrue())
-		Expect(StringInSlice("homer.com", pod.Spec.DNSConfig.Searches)).To(BeTrue())
+		Expect(pod.Spec.DNSConfig.Nameservers).To(ContainElement("8.8.8.8"))
+		Expect(pod.Spec.DNSConfig.Nameservers).To(ContainElement("8.7.7.7"))
+		Expect(pod.Spec.DNSConfig.Searches).To(ContainElement("foobar.com"))
+		Expect(pod.Spec.DNSConfig.Searches).To(ContainElement("homer.com"))
 	})
 
 	It("podman generate kube on a pod with dns options", func() {
@@ -809,7 +959,7 @@ var _ = Describe("Podman generate kube", func() {
 		top.WaitWithDefaultTimeout()
 		Expect(top).Should(Exit(0))
 
-		kube := podmanTest.Podman([]string{"generate", "kube", "pod1"})
+		kube := podmanTest.Podman([]string{"kube", "generate", "pod1"})
 		kube.WaitWithDefaultTimeout()
 		Expect(kube).Should(Exit(0))
 
@@ -817,11 +967,12 @@ var _ = Describe("Podman generate kube", func() {
 		err := yaml.Unmarshal(kube.Out.Contents(), pod)
 		Expect(err).To(BeNil())
 
-		Expect(StringInSlice("8.8.8.8", pod.Spec.DNSConfig.Nameservers)).To(BeTrue())
-		Expect(StringInSlice("foobar.com", pod.Spec.DNSConfig.Searches)).To(BeTrue())
+		Expect(pod.Spec.DNSConfig.Nameservers).To(ContainElement("8.8.8.8"))
+		Expect(pod.Spec.DNSConfig.Searches).To(ContainElement("foobar.com"))
 		Expect(len(pod.Spec.DNSConfig.Options)).To(BeNumerically(">", 0))
-		Expect(pod.Spec.DNSConfig.Options[0].Name).To(Equal("color"))
-		Expect(*pod.Spec.DNSConfig.Options[0].Value).To(Equal("blue"))
+		Expect(pod.Spec.DNSConfig.Options[0]).To(HaveField("Name", "color"))
+		s := "blue"
+		Expect(pod.Spec.DNSConfig.Options[0]).To(HaveField("Value", &s))
 	})
 
 	It("podman generate kube - set entrypoint as command", func() {
@@ -840,10 +991,10 @@ var _ = Describe("Podman generate kube", func() {
 		Expect(err).To(BeNil())
 
 		containers := pod.Spec.Containers
-		Expect(len(containers)).To(Equal(1))
+		Expect(containers).To(HaveLen(1))
 
-		Expect(containers[0].Command).To(Equal([]string{"/bin/sleep"}))
-		Expect(containers[0].Args).To(Equal([]string{"10s"}))
+		Expect(containers[0]).To(HaveField("Command", []string{"/bin/sleep"}))
+		Expect(containers[0]).To(HaveField("Args", []string{"10s"}))
 	})
 
 	It("podman generate kube - use command from image unless explicitly set in the podman command", func() {
@@ -862,15 +1013,15 @@ var _ = Describe("Podman generate kube", func() {
 		Expect(err).To(BeNil())
 
 		containers := pod.Spec.Containers
-		Expect(len(containers)).To(Equal(1))
-		Expect(len(containers[0].Command)).To(Equal(0))
+		Expect(containers).To(HaveLen(1))
+		Expect(containers[0].Command).To(BeEmpty())
 
 		cmd := []string{"echo", "hi"}
 		session = podmanTest.Podman(append([]string{"create", "--name", "test1", ALPINE}, cmd...))
 		session.WaitWithDefaultTimeout()
 		Expect(session).Should(Exit(0))
 
-		kube = podmanTest.Podman([]string{"generate", "kube", "test1"})
+		kube = podmanTest.Podman([]string{"kube", "generate", "test1"})
 		kube.WaitWithDefaultTimeout()
 		Expect(kube).Should(Exit(0))
 
@@ -881,8 +1032,8 @@ var _ = Describe("Podman generate kube", func() {
 		Expect(err).To(BeNil())
 
 		containers = pod.Spec.Containers
-		Expect(len(containers)).To(Equal(1))
-		Expect(containers[0].Command).To(Equal(cmd))
+		Expect(containers).To(HaveLen(1))
+		Expect(containers[0]).To(HaveField("Command", cmd))
 	})
 
 	It("podman generate kube - use entrypoint from image unless --entrypoint is set", func() {
@@ -893,7 +1044,7 @@ ENTRYPOINT ["sleep"]`
 		targetPath, err := CreateTempDirInTempDir()
 		Expect(err).To(BeNil())
 		containerfilePath := filepath.Join(targetPath, "Containerfile")
-		err = ioutil.WriteFile(containerfilePath, []byte(containerfile), 0644)
+		err = os.WriteFile(containerfilePath, []byte(containerfile), 0644)
 		Expect(err).To(BeNil())
 
 		image := "generatekube:test"
@@ -916,14 +1067,14 @@ ENTRYPOINT ["sleep"]`
 		Expect(err).To(BeNil())
 
 		containers := pod.Spec.Containers
-		Expect(len(containers)).To(Equal(1))
-		Expect(containers[0].Args).To(Equal([]string{"10s"}))
+		Expect(containers).To(HaveLen(1))
+		Expect(containers[0]).To(HaveField("Args", []string{"10s"}))
 
 		session = podmanTest.Podman([]string{"create", "--pod", "new:testpod-2", "--entrypoint", "echo", image, "hello"})
 		session.WaitWithDefaultTimeout()
 		Expect(session).Should(Exit(0))
 
-		kube = podmanTest.Podman([]string{"generate", "kube", "testpod-2"})
+		kube = podmanTest.Podman([]string{"kube", "generate", "testpod-2"})
 		kube.WaitWithDefaultTimeout()
 		Expect(kube).Should(Exit(0))
 
@@ -934,9 +1085,9 @@ ENTRYPOINT ["sleep"]`
 		Expect(err).To(BeNil())
 
 		containers = pod.Spec.Containers
-		Expect(len(containers)).To(Equal(1))
-		Expect(containers[0].Command).To(Equal([]string{"echo"}))
-		Expect(containers[0].Args).To(Equal([]string{"hello"}))
+		Expect(containers).To(HaveLen(1))
+		Expect(containers[0]).To(HaveField("Command", []string{"echo"}))
+		Expect(containers[0]).To(HaveField("Args", []string{"hello"}))
 	})
 
 	It("podman generate kube - --privileged container", func() {
@@ -954,7 +1105,7 @@ ENTRYPOINT ["sleep"]`
 		Expect(err).To(BeNil())
 
 		containers := pod.Spec.Containers
-		Expect(len(containers)).To(Equal(1))
+		Expect(containers).To(HaveLen(1))
 		Expect(containers[0].SecurityContext.Capabilities).To(BeNil())
 
 		// Now make sure we can also `play` it.
@@ -965,7 +1116,7 @@ ENTRYPOINT ["sleep"]`
 		Expect(kube).Should(Exit(0))
 
 		// Remove the pod so play can recreate it.
-		kube = podmanTest.Podman([]string{"pod", "rm", "-f", "testpod"})
+		kube = podmanTest.Podman([]string{"pod", "rm", "-t", "0", "-f", "testpod"})
 		kube.WaitWithDefaultTimeout()
 		Expect(kube).Should(Exit(0))
 
@@ -983,7 +1134,7 @@ USER test1`
 		targetPath, err := CreateTempDirInTempDir()
 		Expect(err).To(BeNil())
 		containerfilePath := filepath.Join(targetPath, "Containerfile")
-		err = ioutil.WriteFile(containerfilePath, []byte(containerfile), 0644)
+		err = os.WriteFile(containerfilePath, []byte(containerfile), 0644)
 		Expect(err).To(BeNil())
 
 		image := "generatekube:test"
@@ -1019,7 +1170,7 @@ USER test1`
 		pvc := new(v1.PersistentVolumeClaim)
 		err := yaml.Unmarshal(kube.Out.Contents(), pvc)
 		Expect(err).To(BeNil())
-		Expect(pvc.GetName()).To(Equal(vol))
+		Expect(pvc).To(HaveField("Name", vol))
 		Expect(pvc.Spec.AccessModes[0]).To(Equal(v1.ReadWriteOnce))
 		Expect(pvc.Spec.Resources.Requests.Storage().String()).To(Equal("1Gi"))
 	})
@@ -1041,11 +1192,11 @@ USER test1`
 		pvc := new(v1.PersistentVolumeClaim)
 		err := yaml.Unmarshal(kube.Out.Contents(), pvc)
 		Expect(err).To(BeNil())
-		Expect(pvc.GetName()).To(Equal(vol))
+		Expect(pvc).To(HaveField("Name", vol))
 		Expect(pvc.Spec.AccessModes[0]).To(Equal(v1.ReadWriteOnce))
 		Expect(pvc.Spec.Resources.Requests.Storage().String()).To(Equal("1Gi"))
 
-		for k, v := range pvc.GetAnnotations() {
+		for k, v := range pvc.Annotations {
 			switch k {
 			case util.VolumeDeviceAnnotation:
 				Expect(v).To(Equal(volDevice))
@@ -1070,9 +1221,7 @@ USER test1`
 		err := yaml.Unmarshal(kube.Out.Contents(), pod)
 		Expect(err).To(BeNil())
 
-		v, ok := pod.GetAnnotations()["io.containers.autoupdate/top"]
-		Expect(ok).To(Equal(true))
-		Expect(v).To(Equal("local"))
+		Expect(pod.Annotations).To(HaveKeyWithValue("io.containers.autoupdate/top", "local"))
 	})
 
 	It("podman generate kube on pod with auto update labels in all containers", func() {
@@ -1095,17 +1244,59 @@ USER test1`
 		pod := new(v1.Pod)
 		err := yaml.Unmarshal(kube.Out.Contents(), pod)
 		Expect(err).To(BeNil())
-		Expect(pod.Spec.Containers[0].WorkingDir).To(Equal(""))
-		Expect(pod.Spec.Containers[1].WorkingDir).To(Equal("/root"))
+		Expect(pod.Spec.Containers[0]).To(HaveField("WorkingDir", ""))
+		Expect(pod.Spec.Containers[1]).To(HaveField("WorkingDir", "/root"))
 
 		for _, ctr := range []string{"top1", "top2"} {
-			v, ok := pod.GetAnnotations()["io.containers.autoupdate/"+ctr]
-			Expect(ok).To(Equal(true))
-			Expect(v).To(Equal("registry"))
-
-			v, ok = pod.GetAnnotations()["io.containers.autoupdate.authfile/"+ctr]
-			Expect(ok).To(Equal(true))
-			Expect(v).To(Equal("/some/authfile.json"))
+			Expect(pod.Annotations).To(HaveKeyWithValue("io.containers.autoupdate/"+ctr, "registry"))
+			Expect(pod.Annotations).To(HaveKeyWithValue("io.containers.autoupdate.authfile/"+ctr, "/some/authfile.json"))
 		}
 	})
+
+	It("podman generate kube can export env variables correctly", func() {
+		// Fixes https://github.com/containers/podman/issues/12647
+		// PR https://github.com/containers/podman/pull/12648
+
+		ctrName := "gen-kube-env-ctr"
+		podName := "gen-kube-env"
+		session1 := podmanTest.Podman([]string{"run", "-d", "--pod", "new:" + podName, "--name", ctrName,
+			"-e", "FOO=bar",
+			"-e", "HELLO=WORLD",
+			"alpine", "top"})
+		session1.WaitWithDefaultTimeout()
+		Expect(session1).Should(Exit(0))
+
+		kube := podmanTest.Podman([]string{"generate", "kube", podName})
+		kube.WaitWithDefaultTimeout()
+		Expect(kube).Should(Exit(0))
+
+		pod := new(v1.Pod)
+		err := yaml.Unmarshal(kube.Out.Contents(), pod)
+		Expect(err).To(BeNil())
+
+		Expect(pod.Spec.Containers[0].Env).To(HaveLen(2))
+	})
+
+	It("podman generate kube omit secret if empty", func() {
+		dir, err := os.MkdirTemp(tempdir, "podman")
+		Expect(err).Should(BeNil())
+
+		defer os.RemoveAll(dir)
+
+		podCreate := podmanTest.Podman([]string{"run", "-d", "--pod", "new:" + "noSecretsPod", "--name", "noSecretsCtr", "--volume", dir + ":/foobar", ALPINE})
+		podCreate.WaitWithDefaultTimeout()
+		Expect(podCreate).Should(Exit(0))
+
+		kube := podmanTest.Podman([]string{"generate", "kube", "noSecretsPod"})
+		kube.WaitWithDefaultTimeout()
+		Expect(kube).Should(Exit(0))
+
+		Expect(kube.OutputToString()).ShouldNot(ContainSubstring("secret"))
+
+		pod := new(v1.Pod)
+		err = yaml.Unmarshal(kube.Out.Contents(), pod)
+		Expect(err).To(BeNil())
+
+		Expect(pod.Spec.Volumes[0].Secret).To(BeNil())
+	})
 })
