@@ -7,7 +7,8 @@ import (
 	"strings"
 	"time"
 
-	. "github.com/containers/podman/v3/test/utils"
+	. "github.com/containers/podman/v4/test/utils"
+	"github.com/containers/storage/pkg/stringid"
 	. "github.com/onsi/ginkgo"
 	. "github.com/onsi/gomega"
 	. "github.com/onsi/gomega/gexec"
@@ -20,10 +21,7 @@ func isEventBackendJournald(podmanTest *PodmanTestIntegration) bool {
 	}
 	info := podmanTest.Podman([]string{"info", "--format", "{{.Host.EventLogger}}"})
 	info.WaitWithDefaultTimeout()
-	if info.OutputToString() == "journald" {
-		return true
-	}
-	return false
+	return info.OutputToString() == "journald"
 }
 
 var _ = Describe("Podman logs", func() {
@@ -40,7 +38,6 @@ var _ = Describe("Podman logs", func() {
 		}
 		podmanTest = PodmanTestCreate(tempdir)
 		podmanTest.Setup()
-		podmanTest.SeedImages()
 	})
 
 	AfterEach(func() {
@@ -71,7 +68,7 @@ var _ = Describe("Podman logs", func() {
 			results := podmanTest.Podman([]string{"logs", cid})
 			results.WaitWithDefaultTimeout()
 			Expect(results).To(Exit(0))
-			Expect(len(results.OutputToStringArray())).To(Equal(3))
+			Expect(results.OutputToStringArray()).To(HaveLen(3))
 			Expect(results.OutputToString()).To(Equal("podman podman podman"))
 		})
 
@@ -86,7 +83,7 @@ var _ = Describe("Podman logs", func() {
 			results := podmanTest.Podman([]string{"logs", "--tail", "2", cid})
 			results.WaitWithDefaultTimeout()
 			Expect(results).To(Exit(0))
-			Expect(len(results.OutputToStringArray())).To(Equal(2))
+			Expect(results.OutputToStringArray()).To(HaveLen(2))
 		})
 
 		It("tail zero lines: "+log, func() {
@@ -100,35 +97,41 @@ var _ = Describe("Podman logs", func() {
 			results := podmanTest.Podman([]string{"logs", "--tail", "0", cid})
 			results.WaitWithDefaultTimeout()
 			Expect(results).To(Exit(0))
-			Expect(len(results.OutputToStringArray())).To(Equal(0))
+			Expect(results.OutputToStringArray()).To(BeEmpty())
 		})
 
 		It("tail 99 lines: "+log, func() {
 			skipIfJournaldInContainer()
 
-			logc := podmanTest.Podman([]string{"run", "--log-driver", log, "-dt", ALPINE, "sh", "-c", "echo podman; echo podman; echo podman"})
+			name := "test1"
+			logc := podmanTest.Podman([]string{"run", "--name", name, "--log-driver", log, ALPINE, "sh", "-c", "echo podman; echo podman; echo podman"})
 			logc.WaitWithDefaultTimeout()
 			Expect(logc).To(Exit(0))
-			cid := logc.OutputToString()
 
-			results := podmanTest.Podman([]string{"logs", "--tail", "99", cid})
+			results := podmanTest.Podman([]string{"logs", "--tail", "99", name})
 			results.WaitWithDefaultTimeout()
 			Expect(results).To(Exit(0))
-			Expect(len(results.OutputToStringArray())).To(Equal(3))
+			Expect(results.OutputToStringArray()).To(HaveLen(3))
 		})
 
 		It("tail 800 lines: "+log, func() {
 			skipIfJournaldInContainer()
 
+			// this uses -d so that we do not have 1000 unnecessary lines printed in every test log
 			logc := podmanTest.Podman([]string{"run", "--log-driver", log, "-dt", ALPINE, "sh", "-c", "i=1; while [ \"$i\" -ne 1000 ]; do echo \"line $i\"; i=$((i + 1)); done"})
 			logc.WaitWithDefaultTimeout()
 			Expect(logc).To(Exit(0))
 			cid := logc.OutputToString()
 
+			// make sure we wait for the container to finish writing its output
+			wait := podmanTest.Podman([]string{"wait", cid})
+			wait.WaitWithDefaultTimeout()
+			Expect(wait).To(Exit(0))
+
 			results := podmanTest.Podman([]string{"logs", "--tail", "800", cid})
 			results.WaitWithDefaultTimeout()
 			Expect(results).To(Exit(0))
-			Expect(len(results.OutputToStringArray())).To(Equal(800))
+			Expect(results.OutputToStringArray()).To(HaveLen(800))
 		})
 
 		It("tail 2 lines with timestamps: "+log, func() {
@@ -142,7 +145,7 @@ var _ = Describe("Podman logs", func() {
 			results := podmanTest.Podman([]string{"logs", "--tail", "2", "-t", cid})
 			results.WaitWithDefaultTimeout()
 			Expect(results).To(Exit(0))
-			Expect(len(results.OutputToStringArray())).To(Equal(2))
+			Expect(results.OutputToStringArray()).To(HaveLen(2))
 		})
 
 		It("since time 2017-08-07: "+log, func() {
@@ -156,7 +159,7 @@ var _ = Describe("Podman logs", func() {
 			results := podmanTest.Podman([]string{"logs", "--since", "2017-08-07T10:10:09.056611202-04:00", cid})
 			results.WaitWithDefaultTimeout()
 			Expect(results).To(Exit(0))
-			Expect(len(results.OutputToStringArray())).To(Equal(3))
+			Expect(results.OutputToStringArray()).To(HaveLen(3))
 		})
 
 		It("since duration 10m: "+log, func() {
@@ -170,7 +173,7 @@ var _ = Describe("Podman logs", func() {
 			results := podmanTest.Podman([]string{"logs", "--since", "10m", cid})
 			results.WaitWithDefaultTimeout()
 			Expect(results).To(Exit(0))
-			Expect(len(results.OutputToStringArray())).To(Equal(3))
+			Expect(results.OutputToStringArray()).To(HaveLen(3))
 		})
 
 		It("until duration 10m: "+log, func() {
@@ -184,7 +187,7 @@ var _ = Describe("Podman logs", func() {
 			results := podmanTest.Podman([]string{"logs", "--until", "10m", cid})
 			results.WaitWithDefaultTimeout()
 			Expect(results).To(Exit(0))
-			Expect(len(results.OutputToStringArray())).To(Equal(3))
+			Expect(results.OutputToStringArray()).To(HaveLen(3))
 		})
 
 		It("until time NOW: "+log, func() {
@@ -201,7 +204,7 @@ var _ = Describe("Podman logs", func() {
 			results := podmanTest.Podman([]string{"logs", "--until", nowS, cid})
 			results.WaitWithDefaultTimeout()
 			Expect(results).To(Exit(0))
-			Expect(len(results.OutputToStringArray())).To(Equal(3))
+			Expect(results.OutputToStringArray()).To(HaveLen(3))
 		})
 
 		It("latest and container name should fail: "+log, func() {
@@ -214,7 +217,7 @@ var _ = Describe("Podman logs", func() {
 
 		It("two containers showing short container IDs: "+log, func() {
 			skipIfJournaldInContainer()
-			SkipIfRemote("FIXME: podman-remote logs does not support showing two containers at the same time")
+			SkipIfRemote("podman-remote logs does not support showing two containers at the same time")
 
 			log1 := podmanTest.Podman([]string{"run", "--log-driver", log, "-dt", ALPINE, "sh", "-c", "echo podman; echo podman; echo podman"})
 			log1.WaitWithDefaultTimeout()
@@ -231,7 +234,7 @@ var _ = Describe("Podman logs", func() {
 			Expect(results).Should(Exit(0))
 
 			output := results.OutputToStringArray()
-			Expect(len(output)).To(Equal(6))
+			Expect(output).To(HaveLen(6))
 			Expect(strings.Contains(output[0], cid1[:12]) || strings.Contains(output[0], cid2[:12])).To(BeTrue())
 		})
 
@@ -283,7 +286,7 @@ var _ = Describe("Podman logs", func() {
 				Expect(inspect.ErrorToString()).To(ContainSubstring("no such container"))
 			}
 
-			results = podmanTest.Podman([]string{"rm", "-f", containerName})
+			results = podmanTest.Podman([]string{"rm", "--time", "0", "-f", containerName})
 			results.WaitWithDefaultTimeout()
 			Expect(results).To(Exit(0))
 		})
@@ -345,7 +348,7 @@ var _ = Describe("Podman logs", func() {
 			results.WaitWithDefaultTimeout()
 			Expect(results).To(Exit(0))
 			outlines := results.OutputToStringArray()
-			Expect(len(outlines)).To(Equal(2))
+			Expect(outlines).To(HaveLen(2))
 			Expect(outlines[0]).To(Equal("1\r"))
 			Expect(outlines[1]).To(Equal("2\r"))
 		})
@@ -368,6 +371,98 @@ var _ = Describe("Podman logs", func() {
 			Expect(results.OutputToString()).To(Equal("stdout"))
 			Expect(results.ErrorToString()).To(Equal("stderr"))
 		})
+
+		It("podman logs partial log lines: "+log, func() {
+			skipIfJournaldInContainer()
+
+			cname := "log-test"
+			content := stringid.GenerateRandomID()
+			// use printf to print no extra newline
+			logc := podmanTest.Podman([]string{"run", "--log-driver", log, "--name", cname, ALPINE, "printf", content})
+			logc.WaitWithDefaultTimeout()
+			Expect(logc).To(Exit(0))
+			// Important: do not use OutputToString(), this will remove the trailing newline from the output.
+			// However this test must make sure that there is no such extra newline.
+			Expect(string(logc.Out.Contents())).To(Equal(content))
+
+			logs := podmanTest.Podman([]string{"logs", cname})
+			logs.WaitWithDefaultTimeout()
+			Expect(logs).To(Exit(0))
+			// see comment above
+			Expect(string(logs.Out.Contents())).To(Equal(content))
+		})
+
+		It("podman pod logs -l with newer container created: "+log, func() {
+			skipIfJournaldInContainer()
+
+			podName := "testPod"
+			containerName1 := "container1"
+			containerName2 := "container2"
+			containerName3 := "container3"
+
+			testPod := podmanTest.Podman([]string{"pod", "create", fmt.Sprintf("--name=%s", podName)})
+			testPod.WaitWithDefaultTimeout()
+			Expect(testPod).To(Exit(0))
+
+			log1 := podmanTest.Podman([]string{"run", "--log-driver", log, "--name", containerName1, "-d", "--pod", podName, BB, "/bin/sh", "-c", "echo log1"})
+			log1.WaitWithDefaultTimeout()
+			Expect(log1).To(Exit(0))
+
+			log2 := podmanTest.Podman([]string{"run", "--log-driver", log, "--name", containerName2, "-d", "--pod", podName, BB, "/bin/sh", "-c", "echo log2"})
+			log2.WaitWithDefaultTimeout()
+			Expect(log2).To(Exit(0))
+
+			ctr := podmanTest.Podman([]string{"run", "--log-driver", log, "--name", containerName3, "-d", BB, "date"})
+			ctr.WaitWithDefaultTimeout()
+			Expect(ctr).To(Exit(0))
+
+			results := podmanTest.Podman([]string{"pod", "logs", "-l"})
+			results.WaitWithDefaultTimeout()
+			if IsRemote() {
+				Expect(results).To(Exit(125))
+			} else {
+				Expect(results).To(Exit(0))
+				podOutput := results.OutputToString()
+
+				results = podmanTest.Podman([]string{"logs", "-l"})
+				results.WaitWithDefaultTimeout()
+				Expect(results).To(Exit(0))
+				ctrOutput := results.OutputToString()
+
+				Expect(podOutput).ToNot(Equal(ctrOutput))
+			}
+		})
+
+		It("podman pod logs -l: "+log, func() {
+			skipIfJournaldInContainer()
+
+			podName := "testPod"
+			containerName1 := "container1"
+			containerName2 := "container2"
+
+			testPod := podmanTest.Podman([]string{"pod", "create", fmt.Sprintf("--name=%s", podName)})
+			testPod.WaitWithDefaultTimeout()
+			Expect(testPod).To(Exit(0))
+
+			log1 := podmanTest.Podman([]string{"run", "--log-driver", log, "--name", containerName1, "-d", "--pod", podName, BB, "/bin/sh", "-c", "echo log1"})
+			log1.WaitWithDefaultTimeout()
+			Expect(log1).To(Exit(0))
+
+			log2 := podmanTest.Podman([]string{"run", "--log-driver", log, "--name", containerName2, "-d", "--pod", podName, BB, "/bin/sh", "-c", "echo log2"})
+			log2.WaitWithDefaultTimeout()
+			Expect(log2).To(Exit(0))
+
+			results := podmanTest.Podman([]string{"pod", "logs", "-l"})
+			results.WaitWithDefaultTimeout()
+			if IsRemote() {
+				Expect(results).To(Exit(125))
+			} else {
+				Expect(results).To(Exit(0))
+				output := results.OutputToString()
+				Expect(output).To(ContainSubstring("log1"))
+				Expect(output).To(ContainSubstring("log2"))
+			}
+		})
 	}
 
 	It("using journald for container with container tag", func() {
@@ -415,4 +510,57 @@ var _ = Describe("Podman logs", func() {
 		logs.WaitWithDefaultTimeout()
 		Expect(logs).To(Not(Exit(0)))
 	})
+
+	It("podman pod logs with container names", func() {
+		SkipIfRemote("Remote can only process one container at a time")
+		SkipIfInContainer("journalctl inside a container doesn't work correctly")
+		podName := "testPod"
+		containerName1 := "container1"
+		containerName2 := "container2"
+
+		testPod := podmanTest.Podman([]string{"pod", "create", fmt.Sprintf("--name=%s", podName)})
+		testPod.WaitWithDefaultTimeout()
+		Expect(testPod).To(Exit(0))
+
+		log1 := podmanTest.Podman([]string{"run", "--name", containerName1, "-d", "--pod", podName, BB, "/bin/sh", "-c", "echo log1"})
+		log1.WaitWithDefaultTimeout()
+		Expect(log1).To(Exit(0))
+
+		log2 := podmanTest.Podman([]string{"run", "--name", containerName2, "-d", "--pod", podName, BB, "/bin/sh", "-c", "echo log2"})
+		log2.WaitWithDefaultTimeout()
+		Expect(log2).To(Exit(0))
+
+		results := podmanTest.Podman([]string{"pod", "logs", "--names", podName})
+		results.WaitWithDefaultTimeout()
+		Expect(results).To(Exit(0))
+
+		output := results.OutputToStringArray()
+		Expect(output).To(HaveLen(2))
+		Expect(output).To(ContainElement(ContainSubstring(containerName1)))
+		Expect(output).To(ContainElement(ContainSubstring(containerName2)))
+	})
+	It("podman pod logs with different colors", func() {
+		SkipIfRemote("Remote can only process one container at a time")
+		SkipIfInContainer("journalctl inside a container doesn't work correctly")
+		podName := "testPod"
+		containerName1 := "container1"
+		containerName2 := "container2"
+		testPod := podmanTest.Podman([]string{"pod", "create", fmt.Sprintf("--name=%s", podName)})
+		testPod.WaitWithDefaultTimeout()
+		Expect(testPod).To(Exit(0))
+		log1 := podmanTest.Podman([]string{"run", "--name", containerName1, "-d", "--pod", podName, BB, "/bin/sh", "-c", "echo log1"})
+		log1.WaitWithDefaultTimeout()
+		Expect(log1).To(Exit(0))
+		log2 := podmanTest.Podman([]string{"run", "--name", containerName2, "-d", "--pod", podName, BB, "/bin/sh", "-c", "echo log2"})
+		log2.WaitWithDefaultTimeout()
+		Expect(log2).To(Exit(0))
+		results := podmanTest.Podman([]string{"pod", "logs", "--color", podName})
+		results.WaitWithDefaultTimeout()
+		Expect(results).To(Exit(0))
+		output := results.OutputToStringArray()
+		Expect(output).To(HaveLen(2))
+		Expect(output[0]).To(MatchRegexp(`\x1b\[3[0-9a-z ]+\x1b\[0m`))
+		Expect(output[1]).To(MatchRegexp(`\x1b\[3[0-9a-z ]+\x1b\[0m`))
+	})
+
 })
